define(["require", "exports", 'common/appUrl', 'models/changeSubscription', 'common/changesCallback', 'commands/commandBase'], function(require, exports, appUrl, changeSubscription, changesCallback, commandBase) {
    var changesApi = (function () {
        function changesApi(db) {
            this.db = db;
            this.allDocsHandlers = ko.observableArray();
            this.allIndexesHandlers = ko.observableArray();
            this.commandBase = new commandBase();
            this.eventsId = this.makeid();
            this.connect();
        }
        changesApi.prototype.connect = function () {
            var _this = this;
            if (!!window.EventSource) {
                var dbUrl = appUrl.forResourceQuery(this.db);

                this.source = new EventSource(dbUrl + '/changes/events?id=' + this.eventsId);
                this.source.onmessage = function (e) {
                    return _this.onEvent(e);
                };
                this.source.onerror = function (e) {
                    return _this.onError(e);
                };
            } else {
                console.log("EventSource is not supported");
            }
        };

        changesApi.prototype.send = function (command, value) {
            var args = {
                id: this.eventsId,
                command: command
            };
            if (value !== undefined) {
                args["value"] = value;
            }

            this.commandBase.query('/changes/config', args, this.db);
        };

        changesApi.prototype.onError = function (e) {
            this.commandBase.reportError('Changes stream was disconnected. Retrying connection shortly.');
        };

        changesApi.prototype.fireEvents = function (events, param) {
            for (var i = 0; i < events.length; i++) {
                events[i].fire(param);
            }
        };

        changesApi.prototype.onEvent = function (e) {
            var json = JSON.parse(e.data);
            var type = json.Type;
            if (type === "Heartbeat") {
            } else if (type === "DocumentChangeNotification") {
                this.fireEvents(this.allDocsHandlers(), json.Value);
            } else if (type === "IndexChangeNotification") {
                this.fireEvents(this.allIndexesHandlers(), json.Value);
            } else {
                console.log("Unhandled notification type: " + type);
            }
        };

        changesApi.prototype.watchAllIndexes = function (onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            if (this.allIndexesHandlers().length == 0) {
                this.send('watch-indexes');
            }
            this.allIndexesHandlers.push(callback);
            return new changeSubscription(function () {
                _this.allIndexesHandlers.remove(callback);
                if (_this.allIndexesHandlers().length == 0) {
                    _this.send('unwatch-indexes');
                }
            });
        };

        changesApi.prototype.watchAllDocs = function (onChange) {
            var _this = this;
            var callback = new changesCallback(onChange);
            if (this.allDocsHandlers().length == 0) {
                this.send('watch-docs');
            }
            this.allDocsHandlers.push(callback);
            return new changeSubscription(function () {
                _this.allDocsHandlers.remove(callback);
                if (_this.allDocsHandlers().length == 0) {
                    _this.send('unwatch-docs');
                }
            });
        };

        changesApi.prototype.watchDocPrefix = function (onChange, prefix) {
            var _this = this;
            var callback = new changesCallback(onChange);
            if (this.allDocsHandlers().length == 0) {
                this.send('watch-prefix', prefix);
            }
            this.allDocsHandlers.push(callback);
            return new changeSubscription(function () {
                _this.allDocsHandlers.remove(callback);
                if (_this.allDocsHandlers().length == 0) {
                    _this.send('watch-prefix', prefix);
                }
            });
        };

        changesApi.prototype.dispose = function () {
            if (this.source) {
                this.send('disconnect');
                this.source.close();
            }
        };

        changesApi.prototype.makeid = function () {
            var text = "";
            var chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";

            for (var i = 0; i < 5; i++)
                text += chars.charAt(Math.floor(Math.random() * chars.length));

            return text;
        };
        return changesApi;
    })();

    
    return changesApi;
});
