var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "models/databaseAccess", "common/appUrl", "models/documentMetadata", "models/document"], function(require, exports, databaseAccess, appUrl, documentMetadata, document) {
    var apiKey = (function (_super) {
        __extends(apiKey, _super);
        function apiKey(dto) {
            var _this = this;
            _super.call(this, dto);
            this.name = ko.observable();
            this.secret = ko.observable();
            this.enabled = ko.observable();
            this.databases = ko.observableArray();
            this.visible = ko.observable(true);

            this.name(dto.Name);
            this.secret(dto.Secret);
            this.enabled(dto.Enabled);
            this.databases(dto.Databases.map(function (d) {
                return new databaseAccess(d);
            }));
            this.metadata = new documentMetadata(dto['@metadata']);

            this.fullApiKey = ko.computed(function () {
                if (!_this.name() || !_this.secret()) {
                    return "Requires name and secret";
                }

                return _this.name() + "/" + _this.secret();
            });

            this.connectionString = ko.computed(function () {
                if (!_this.fullApiKey()) {
                    return "Requires name and secret";
                }

                return "Url = " + appUrl.forServer() + "; ApiKey = " + _this.fullApiKey() + "; Database = ";
            });

            this.directLink = ko.computed(function () {
                if (!_this.fullApiKey()) {
                    return "Requires name and secret";
                }

                return appUrl.forServer() + "/raven/studio.html#/home?api-key=" + _this.fullApiKey();
            });
        }
        apiKey.empty = function () {
            return new apiKey({
                Databases: [],
                Enabled: false,
                Name: "",
                Secret: ""
            });
        };

        apiKey.prototype.toDto = function () {
            var meta = this.__metadata.toDto();
            meta['@id'] = "Raven/ApiKeys/" + this.name();
            return {
                '@metadata': meta,
                Databases: this.databases().map(function (db) {
                    return db.toDto();
                }),
                Enabled: this.enabled(),
                Name: this.name(),
                Secret: this.secret()
            };
        };

        apiKey.prototype.enable = function () {
            this.enabled(true);
        };

        apiKey.prototype.disable = function () {
            this.enabled(false);
        };

        apiKey.prototype.generateSecret = function () {
            var minimumLength = 10;
            var maxLength = 32;
            var randomLength = Math.max(minimumLength, Math.random() * maxLength);
            var randomSecret = apiKey.randomString(randomLength, '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ');
            this.secret(randomSecret);
        };

        apiKey.prototype.addEmptyApiKeyDatabase = function () {
            var newItem = { TenantId: '', Admin: false, ReadOnly: false };
            this.databases.push(new databaseAccess(newItem));
        };

        apiKey.prototype.removeApiKeyDatabase = function (database) {
            this.databases.remove(database);
        };

        apiKey.prototype.setIdFromName = function () {
            this.__metadata.id = "Raven/ApiKeys/" + this.name();
        };

        apiKey.prototype.isValid = function () {
            var requiredValues = [this.name(), this.secret()];
            return requiredValues.every(function (v) {
                return v != null && v.length > 0;
            });
        };

        apiKey.randomString = function (length, chars) {
            var result = '';
            for (var i = length; i > 0; --i)
                result += chars[Math.round(Math.random() * (chars.length - 1))];
            return result;
        };

        apiKey.prototype.getKey = function () {
            return "Raven/ApiKeys/" + this.name();
        };
        return apiKey;
    })(document);

    
    return apiKey;
});
