define(["require", "exports", "models/documentMetadata"], function(require, exports, documentMetadata) {
    var document = (function () {
        function document(dto) {
            this.__metadata = new documentMetadata(dto['@metadata']);
            for (var property in dto) {
                if (property !== '@metadata') {
                    this[property] = dto[property];
                }
            }
        }
        document.prototype.getEntityName = function () {
            return this.__metadata.ravenEntityName;
        };

        document.prototype.getId = function () {
            return this.__metadata.id;
        };

        document.prototype.getDocumentPropertyNames = function () {
            var propertyNames = [];
            for (var property in this) {
                var isMeta = property === '__metadata' || property === '__moduleId__';
                var isFunction = typeof this[property] === 'function';
                if (!isMeta && !isFunction) {
                    propertyNames.push(property);
                }
            }

            return propertyNames;
        };

        document.prototype.toDto = function (includeMeta) {
            if (typeof includeMeta === "undefined") { includeMeta = false; }
            var dto = { '@metadata': undefined };
            var properties = this.getDocumentPropertyNames();
            for (var i = 0; i < properties.length; i++) {
                var property = properties[i];
                dto[property] = this[property];
            }

            if (includeMeta && this.__metadata) {
                dto['@metadata'] = this.__metadata.toDto();
            }

            return dto;
        };

        document.prototype.toBulkDoc = function (method) {
            var dto = this.toDto();
            var bulkDoc = {
                Document: dto,
                Key: this.getId(),
                Method: method,
                AdditionalData: null
            };

            var meta = dto["@metadata"];
            if (meta) {
                bulkDoc.Metadata = meta;

                if (meta["@etag"]) {
                    bulkDoc.Etag = meta["@etag"];
                }
            }

            return bulkDoc;
        };

        document.empty = function () {
            var emptyDto = {
                '@metadata': {},
                'Name': '...'
            };

            return new document(emptyDto);
        };

        document.getEntityNameFromId = function (id) {
            if (!id) {
                return null;
            }

            var slashIndex = id.lastIndexOf('/');
            if (slashIndex >= 1) {
                return id.substring(0, 1).toUpperCase() + id.substring(1, slashIndex);
            }

            return id;
        };
        return document;
    })();

    
    return document;
});
