define(["require", "exports", "models/luceneField", "models/spatialIndexField"], function(require, exports, luceneField, spatialIndexField) {
    var indexDefinition = (function () {
        function indexDefinition(dto) {
            this.fields = ko.observableArray().extend({ required: true });
            this.map = ko.observable().extend({ required: true });
            this.maps = ko.observableArray().extend({ required: true });
            this.name = ko.observable().extend({ required: true });
            this.reduce = ko.observable().extend({ required: true });
            this.transformResults = ko.observable().extend({ required: true });
            this.maxIndexOutputsPerDocument = ko.observable(0).extend({ required: true });
            this.luceneFields = ko.observableArray();
            this.spatialFields = ko.observableArray().extend({ required: true });
            this.analyzers = dto.Analyzers;
            this.fields(dto.Fields);
            this.indexes = dto.Indexes;
            this.internalFieldsMapping = dto.InternalFieldsMapping;
            this.isCompiled = dto.IsCompiled;
            this.isMapReduce = dto.IsMapReduce;
            this.lockMode = dto.LockMode;
            this.map(dto.Map);
            this.maps(dto.Maps.map(function (m) {
                return ko.observable(m);
            }));
            this.name(dto.Name);
            this.reduce(dto.Reduce);
            this.sortOptions = dto.SortOptions;
            this.spatialIndexes = dto.SpatialIndexes;
            this.stores = dto.Stores;
            this.suggestions = dto.Suggestions;
            this.termVectors = dto.TermVectors;
            this.transformResults(dto.TransformResults);
            this.type = dto.Type;

            this.luceneFields(this.parseFields());
            this.spatialFields(this.parseSpatialFields());

            this.maxIndexOutputsPerDocument(dto.MaxIndexOutputsPerDocument ? dto.MaxIndexOutputsPerDocument : 0);
        }
        indexDefinition.prototype.toDto = function () {
            return {
                Analyzers: this.makeFieldObject(function (f) {
                    return f.indexing() === "Analyzed";
                }, function (f) {
                    return f.analyzer();
                }),
                Fields: this.fields(),
                Indexes: this.makeFieldObject(function (f) {
                    return f.indexing() !== "Default";
                }, function (f) {
                    return f.indexing();
                }),
                InternalFieldsMapping: this.internalFieldsMapping,
                IsCompiled: this.isCompiled,
                IsMapReduce: this.isMapReduce,
                LockMode: this.lockMode,
                Map: this.maps()[0](),
                Maps: this.maps().map(function (m) {
                    return m();
                }).filter(function (m) {
                    return m && m.length > 0;
                }),
                Name: this.name(),
                Reduce: this.reduce(),
                SortOptions: this.makeFieldObject(function (f) {
                    return f.sort() !== "None";
                }, function (f) {
                    return f.sort();
                }),
                SpatialIndexes: this.makeSpatialIndexesObject(),
                Stores: this.makeFieldObject(function (f) {
                    return f.stores() === "Yes";
                }, function (f) {
                    return f.stores();
                }),
                Suggestions: this.makeFieldObject(function (f) {
                    return f.suggestionDistance() !== "None";
                }, function (f) {
                    return f.toSuggestionDto();
                }),
                TermVectors: this.makeFieldObject(function (f) {
                    return f.termVector() !== "No";
                }, function (f) {
                    return f.termVector();
                }),
                TransformResults: this.transformResults(),
                Type: this.type,
                MaxIndexOutputsPerDocument: this.maxIndexOutputsPerDocument() ? this.maxIndexOutputsPerDocument() > 0 ? this.maxIndexOutputsPerDocument() : null : null
            };
        };

        indexDefinition.empty = function () {
            return new indexDefinition({
                Analyzers: {},
                Fields: [],
                Indexes: {},
                InternalFieldsMapping: {},
                IsCompiled: false,
                IsMapReduce: false,
                LockMode: "Unlock",
                Map: " ",
                Maps: [" "],
                Name: "",
                Reduce: null,
                SortOptions: {},
                SpatialIndexes: {},
                Stores: {},
                Suggestions: {},
                TermVectors: {},
                TransformResults: null,
                Type: "Map",
                MaxIndexOutputsPerDocument: null
            });
        };

        indexDefinition.prototype.makeSpatialIndexesObject = function () {
            var spatialIndexesObj = {};
            this.spatialFields().forEach(function (f) {
                return spatialIndexesObj[f.name()] = f.toDto();
            });
            return spatialIndexesObj;
        };

        indexDefinition.prototype.makeFieldObject = function (filter, selector) {
            var obj = {};
            this.luceneFields().filter(filter).forEach(function (f) {
                return obj[f.name()] = selector(f);
            });
            return obj;
        };

        indexDefinition.prototype.parseFields = function () {
            var _this = this;
            return this.fields().filter(function (name) {
                return _this.analyzers[name] != null || _this.indexes[name] != null || _this.sortOptions[name] != null || _this.stores[name] != null || _this.suggestions[name] != null || _this.termVectors[name] != null;
            }).map(function (fieldName) {
                var suggestion = _this.suggestions && _this.suggestions[fieldName] ? _this.suggestions[fieldName] : {};
                return new luceneField(fieldName, _this.stores[fieldName], _this.indexes[fieldName], _this.sortOptions[fieldName], _this.analyzers[fieldName], suggestion['Distance'], suggestion['Accuracy'], _this.termVectors[fieldName]);
            });
        };

        indexDefinition.prototype.parseSpatialFields = function () {
            var _this = this;
            return this.fields().filter(function (fieldName) {
                return _this.spatialIndexes && _this.spatialIndexes[fieldName];
            }).map(function (fieldName) {
                return new spatialIndexField(fieldName, _this.spatialIndexes[fieldName]);
            });
        };
        return indexDefinition;
    })();

    
    return indexDefinition;
});
