define(["require", "exports", "models/scriptedIndex"], function(require, exports, scriptedIndex) {
    var scriptedIndexMap = (function () {
        function scriptedIndexMap(scriptedIndexes) {
            var _this = this;
            this.PREFIX = 'Raven/ScriptedIndexResults/';
            this.indexes = {};
            this.activeScriptedIndexes = ko.observableArray().extend({ required: true });
            scriptedIndexes.forEach(function (index) {
                _this.indexes[index.__metadata['id']] = index;
                _this.activeScriptedIndexes().push(index);
            });
        }
        scriptedIndexMap.prototype.getIndex = function (indexName) {
            return this.indexes[this.PREFIX + indexName];
        };

        scriptedIndexMap.prototype.addEmptyIndex = function (indexName) {
            var activeScriptedIndex = this.indexes[this.PREFIX + indexName];
            if (activeScriptedIndex) {
                activeScriptedIndex.cancelDeletion();
            } else {
                activeScriptedIndex = scriptedIndex.emptyForIndex(indexName);
                this.indexes[this.PREFIX + indexName] = activeScriptedIndex;
            }
            this.activeScriptedIndexes().push(activeScriptedIndex);
            this.activeScriptedIndexes.notifySubscribers();
        };

        scriptedIndexMap.prototype.removeIndex = function (indexName) {
            var scriptedIndexToDelete = this.indexes[this.PREFIX + indexName];
            scriptedIndexToDelete.markToDelete();

            var index = this.activeScriptedIndexes().indexOf(scriptedIndexToDelete);
            if (index > -1) {
                this.activeScriptedIndexes().splice(index, 1);
                this.activeScriptedIndexes.notifySubscribers();
            }
        };

        scriptedIndexMap.prototype.deleteMarkToDeletedIndex = function (indexName) {
            delete this.indexes[indexName];
        };

        scriptedIndexMap.prototype.getIndexes = function () {
            return jQuery.map(this.indexes, function (value, index) {
                if (value)
                    return value;
            });
        };
        return scriptedIndexMap;
    })();

    
    return scriptedIndexMap;
});
