var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "models/sqlReplicationTable", "models/document", "models/documentMetadata"], function(require, exports, sqlReplicationTable, document, documentMetadata) {
    var sqlReplication = (function (_super) {
        __extends(sqlReplication, _super);
        function sqlReplication(dto) {
            var _this = this;
            _super.call(this, dto);
            this.CONNECTION_STRING = "cs";
            this.CONNECTION_STRING_NAME = "csn";
            this.CONNECTION_STRING_SETTING_NAME = "cssn";
            this.availableConnectionStringTypes = [
                { label: "Connection String", value: this.CONNECTION_STRING },
                { label: "Connection String Name", value: this.CONNECTION_STRING_NAME },
                { label: "Connection String Setting Name", value: this.CONNECTION_STRING_SETTING_NAME }
            ];
            this.name = ko.observable().extend({ required: true });
            this.disabled = ko.observable().extend({ required: true });
            this.factoryName = ko.observable().extend({ required: true });
            this.connectionStringType = ko.observable().extend({ required: true });
            this.connectionStringValue = ko.observable(null).extend({ required: true });
            this.ravenEntityName = ko.observable("").extend({ required: true });
            this.sqlReplicationTables = ko.observableArray().extend({ required: true });
            this.script = ko.observable("").extend({ required: true });
            this.connectionString = ko.observable(null);
            this.connectionStringName = ko.observable(null);
            this.connectionStringSettingName = ko.observable(null);
            this.parameterizeDeletesDisabled = ko.observable();
            this.isFocused = ko.observable(false);

            this.name(dto.Name);
            this.disabled(dto.Disabled);
            this.parameterizeDeletesDisabled(dto.ParameterizeDeletesDisabled);
            this.ravenEntityName(dto.RavenEntityName);
            this.script(dto.Script);
            this.factoryName(dto.FactoryName);
            this.sqlReplicationTables(dto.SqlReplicationTables.map(function (tab) {
                return new sqlReplicationTable(tab);
            }));

            this.setupConnectionString(dto);

            this.metadata = new documentMetadata(dto['@metadata']);

            this.connectionStringSourceFieldName = ko.computed(function () {
                if (_this.connectionStringType() == _this.CONNECTION_STRING) {
                    return "Connection string text";
                } else if (_this.connectionStringType() == _this.CONNECTION_STRING_NAME) {
                    return "Setting name in local machine configuration";
                } else {
                    return "Setting name in memory/remote configuration";
                }
            });
        }
        sqlReplication.prototype.setupConnectionString = function (dto) {
            if (dto.ConnectionStringName) {
                this.connectionStringType(this.CONNECTION_STRING_NAME);
                this.connectionStringValue(dto.ConnectionStringName);
            } else if (dto.ConnectionStringSettingName) {
                this.connectionStringType(this.CONNECTION_STRING_SETTING_NAME);
                this.connectionStringValue(dto.ConnectionStringSettingName);
            } else {
                this.connectionStringType(this.CONNECTION_STRING);
                this.connectionStringValue(dto.ConnectionString);
            }
        };

        sqlReplication.prototype.setConnectionStringType = function (strType) {
            this.connectionStringType(strType);
        };

        sqlReplication.empty = function () {
            var newTable = sqlReplicationTable.empty();
            var sqlReplicationTables = [];
            sqlReplicationTables.push(newTable);
            return new sqlReplication({
                Name: "",
                Disabled: true,
                ParameterizeDeletesDisabled: false,
                RavenEntityName: "",
                Script: "",
                FactoryName: null,
                ConnectionString: null,
                ConnectionStringName: null,
                ConnectionStringSettingName: null,
                SqlReplicationTables: sqlReplicationTables
            });
        };

        sqlReplication.prototype.toDto = function () {
            var meta = this.__metadata.toDto();
            meta['@id'] = "Raven/ApiKeys/" + this.name();
            return {
                '@metadata': meta,
                Name: this.name(),
                Disabled: this.disabled(),
                ParameterizeDeletesDisabled: this.parameterizeDeletesDisabled(),
                RavenEntityName: this.ravenEntityName(),
                Script: this.script(),
                FactoryName: this.factoryName(),
                ConnectionString: this.prepareConnectionString(this.CONNECTION_STRING),
                ConnectionStringName: this.prepareConnectionString(this.CONNECTION_STRING_NAME),
                ConnectionStringSettingName: this.prepareConnectionString(this.CONNECTION_STRING_SETTING_NAME),
                SqlReplicationTables: this.sqlReplicationTables().map(function (tab) {
                    return tab.toDto();
                })
            };
        };

        sqlReplication.prototype.prepareConnectionString = function (expectedType) {
            return ((this.connectionStringType() === expectedType) ? this.connectionStringValue() : null);
        };

        sqlReplication.prototype.enable = function () {
            this.disabled(false);
        };

        sqlReplication.prototype.disable = function () {
            this.disabled(true);
        };

        sqlReplication.prototype.addNewTable = function () {
            this.sqlReplicationTables.push(sqlReplicationTable.empty());
        };

        sqlReplication.prototype.removeTable = function (table) {
            this.sqlReplicationTables.remove(table);
        };

        sqlReplication.prototype.setIdFromName = function () {
            this.__metadata.id = "Raven/SqlReplication/Configuration/" + this.name();
        };

        sqlReplication.prototype.isValid = function () {
            var arr = new Array();

            var requiredValues = [this.name(), this.factoryName(), this.connectionStringType(), this.connectionStringValue(), this.ravenEntityName(), this.script()];
            var fieldsCheck = requiredValues.every(function (v) {
                return v != null && v.length > 0;
            });
            var replicationTablesCheck = this.sqlReplicationTables().every(function (v) {
                return v.isValid();
            });
            return fieldsCheck && replicationTablesCheck;
        };
        return sqlReplication;
    })(document);

    
    return sqlReplication;
});
