var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "commands/getApiKeysCommand", "models/apiKey", "viewmodels/viewModelBase", "commands/getDatabasesCommand", "commands/saveApiKeysCommand"], function(require, exports, getApiKeysCommand, apiKey, viewModelBase, getDatabasesCommand, saveApiKeysCommand) {
    var apiKeys = (function (_super) {
        __extends(apiKeys, _super);
        function apiKeys() {
            var _this = this;
            _super.call(this);
            this.apiKeys = ko.observableArray();
            this.allDatabases = ko.observableArray();
            this.searchText = ko.observable();
            this.hasFetchedApiKeys = ko.observable(false);

            this.areAllApiKeysValid = ko.computed(function () {
                return _this.apiKeys().every(function (k) {
                    return k.isValid();
                });
            });
            this.searchText.throttle(200).subscribe(function (value) {
                return _this.filterKeys(value);
            });
        }
        apiKeys.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);

            this.fetchApiKeys();
            this.fetchDatabases();
        };

        apiKeys.prototype.fetchApiKeys = function () {
            var _this = this;
            new getApiKeysCommand().execute().done(function (results) {
                _this.apiKeys(results);
                _this.hasFetchedApiKeys(true);
            });
        };

        apiKeys.prototype.fetchDatabases = function () {
            var _this = this;
            new getDatabasesCommand().execute().done(function (results) {
                return _this.allDatabases(results.map(function (d) {
                    return d.name;
                }));
            });
        };

        apiKeys.prototype.createNewApiKey = function () {
            this.apiKeys.unshift(apiKey.empty());
        };

        apiKeys.prototype.removeApiKey = function (key) {
            this.apiKeys.remove(key);
        };

        apiKeys.prototype.saveChanges = function () {
            var _this = this;
            this.apiKeys().forEach(function (k) {
                return k.setIdFromName();
            });
            new saveApiKeysCommand(this.apiKeys(), this.activeDatabase()).execute().done(function (result) {
                return _this.updateKeys(result);
            });
        };

        apiKeys.prototype.updateKeys = function (serverKeys) {
            this.apiKeys().forEach(function (key) {
                var serverKey = serverKeys.first(function (k) {
                    return k.Key === key.getId();
                });
                if (serverKey) {
                    key.__metadata.etag = serverKey.Etag;
                    key.__metadata.lastModified = serverKey.Metadata['Last-Modified'];
                }
            });
        };

        apiKeys.prototype.filterKeys = function (filter) {
            var filterLower = filter.toLowerCase().trim();
            this.apiKeys().forEach(function (k) {
                var isEmpty = filterLower.length === 0;
                var isMatch = k.name() != null && k.name().toLowerCase().indexOf(filterLower) !== -1;
                k.visible(isEmpty || isMatch);
            });
        };
        return apiKeys;
    })(viewModelBase);

    
    return apiKeys;
});
