var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "ace/ace", "plugins/router", "common/appUrl", "viewmodels/viewModelBase", "commands/getDatabaseSettingsCommand"], function(require, exports, ace, router, appUrl, viewModelBase, getDatabaseSettingsCommand) {
    var databaseSettings = (function (_super) {
        __extends(databaseSettings, _super);
        function databaseSettings() {
            _super.apply(this, arguments);
            this.documentText = ko.observable('');
        }
        databaseSettings.prototype.attached = function () {
            var _this = this;
            this.initializeDbDocEditor();
            this.fetchDatabaseSettings();
            this.createKeyboardShortcut("F2", function () {
                return _this.navigateToDatabaseSettingDocument();
            }, databaseSettings.containerId);
        };

        databaseSettings.prototype.initializeDbDocEditor = function () {
            this.docEditor = ace.edit("dbDocEditor");
            this.docEditor.setTheme("ace/theme/github");
            this.docEditor.setFontSize("16px");
            this.docEditor.getSession().setMode("ace/mode/json");
            this.docEditor.setReadOnly(true);
        };

        databaseSettings.prototype.fetchDatabaseSettings = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                new getDatabaseSettingsCommand(db).execute().done(function (document) {
                    return _this.handleDocument(document);
                });
            }
        };

        databaseSettings.prototype.getDatabaseSettingsDocumentId = function (db) {
            return "Raven/Databases/" + db.name;
        };

        databaseSettings.prototype.handleDocument = function (doc) {
            var dbSettings = this.stringify(doc.toDto());
            this.docEditor.getSession().setValue(dbSettings);
        };

        databaseSettings.prototype.stringify = function (obj) {
            var prettifySpacing = 4;
            return JSON.stringify(obj, null, prettifySpacing);
        };

        databaseSettings.prototype.navigateToDatabaseSettingDocument = function () {
            var db = this.activeDatabase();
            if (db) {
                var documentId = this.getDatabaseSettingsDocumentId(db);
                var systemDatabase = appUrl.getSystemDatabase();
                var dbSettingsUrl = appUrl.forEditDoc(documentId, null, null, systemDatabase);
                router.navigate(dbSettingsUrl);
            }
        };
        databaseSettings.containerId = "#databaseSettingsContainer";
        return databaseSettings;
    })(viewModelBase);

    
    return databaseSettings;
});
