var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "durandal/app", "plugins/router", "models/collection", "viewmodels/deleteCollection", "common/appUrl", "commands/getCollectionsCommand", 'commands/getCustomColumnsCommand', "viewmodels/viewModelBase", 'models/customColumnParams', 'models/customColumns', 'viewmodels/selectColumns'], function(require, exports, app, router, collection, deleteCollection, appUrl, getCollectionsCommand, getCustomColumnsCommand, viewModelBase, customColumnParams, customColumns, selectColumns) {
    var documents = (function (_super) {
        __extends(documents, _super);
        function documents() {
            var _this = this;
            _super.call(this);
            this.displayName = "documents";
            this.collections = ko.observableArray();
            this.selectedCollection = ko.observable().subscribeTo("ActivateCollection").distinctUntilChanged();
            this.currentCollectionPagedItems = ko.observable();
            this.currentColumnsParams = ko.observable(customColumns.empty());
            this.selectedDocumentIndices = ko.observableArray();
            this.isSelectAll = ko.observable(false);
            this.contextName = ko.observable('');
            this.currentCollection = ko.observable();
            this.selectedCollection.subscribe(function (c) {
                return _this.selectedCollectionChanged(c);
            });
            this.hasAnyDocumentsSelected = ko.computed(function () {
                return _this.selectedDocumentIndices().length > 0;
            });
        }
        documents.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);

            this.collectionToSelectName = args ? args.collection : null;
            this.fetchCollections(appUrl.getDatabase());
        };

        documents.prototype.attached = function () {
            $('.document-collections').contextmenu({
                target: '#collections-context-menu'
            });
        };

        documents.prototype.collectionsLoaded = function (collections, db) {
            var _this = this;
            this.allDocumentsCollection = collection.createAllDocsCollection(db);
            this.allDocumentsCollection.documentCount = ko.computed(function () {
                return _this.collections().filter(function (c) {
                    return c !== _this.allDocumentsCollection;
                }).map(function (c) {
                    return c.documentCount();
                }).reduce(function (first, second) {
                    return first + second;
                }, 0);
            });

            var systemDocumentsCollection = collection.createSystemDocsCollection(db);

            var collectionsWithSysCollection = [systemDocumentsCollection].concat(collections);
            var allCollections = [this.allDocumentsCollection].concat(collectionsWithSysCollection);
            this.collections(allCollections);

            var collectionToSelect = allCollections.first(function (c) {
                return c.name === _this.collectionToSelectName;
            }) || this.allDocumentsCollection;
            collectionToSelect.activate();

            collectionsWithSysCollection.forEach(function (c) {
                return c.fetchTotalDocumentCount();
            });
        };

        documents.prototype.selectedCollectionChanged = function (selected) {
            var _this = this;
            if (selected) {
                var customColumnsCommand = selected.isAllDocuments ? getCustomColumnsCommand.forAllDocuments(this.activeDatabase()) : getCustomColumnsCommand.forCollection(selected.name, this.activeDatabase());

                this.contextName(customColumnsCommand.docName);

                customColumnsCommand.execute().done(function (dto) {
                    if (dto) {
                        _this.currentColumnsParams().columns($.map(dto.Columns, function (c) {
                            return new customColumnParams(c);
                        }));
                        _this.currentColumnsParams().customMode(true);
                    } else {
                        _this.currentColumnsParams().columns.removeAll();
                        _this.currentColumnsParams().customMode(false);
                    }

                    var pagedList = selected.getDocuments();
                    _this.currentCollectionPagedItems(pagedList);
                    _this.currentCollection(selected);
                });
            }
        };

        documents.prototype.deleteCollection = function () {
            var _this = this;
            var collection = this.selectedCollection();
            if (collection) {
                var viewModel = new deleteCollection(collection);
                viewModel.deletionTask.done(function () {
                    _this.collections.remove(collection);
                    _this.allDocumentsCollection.activate();
                });
                app.showDialog(viewModel);
            }
        };

        documents.prototype.selectCollection = function (collection) {
            collection.activate();
            var documentsWithCollectionUrl = appUrl.forDocuments(collection.name, this.activeDatabase());
            router.navigate(documentsWithCollectionUrl, false);
        };

        documents.prototype.selectColumns = function () {
            var _this = this;
            var selectColumnsViewModel = new selectColumns(this.currentColumnsParams().clone(), this.contextName(), this.activeDatabase());
            app.showDialog(selectColumnsViewModel);
            selectColumnsViewModel.onExit().done(function (cols) {
                _this.currentColumnsParams(cols);

                var pagedList = _this.currentCollection().getDocuments();
                _this.currentCollectionPagedItems(pagedList);
            });
        };

        documents.prototype.fetchCollections = function (db) {
            var _this = this;
            return new getCollectionsCommand(db).execute().done(function (results) {
                return _this.collectionsLoaded(results, db);
            });
        };

        documents.prototype.newDocument = function () {
            router.navigate(appUrl.forNewDoc(this.activeDatabase()));
        };

        documents.prototype.toggleSelectAll = function () {
            this.isSelectAll.toggle();

            var docsGrid = this.getDocumentsGrid();
            if (docsGrid && this.isSelectAll()) {
                docsGrid.selectAll();
            } else if (docsGrid && !this.isSelectAll()) {
                docsGrid.selectNone();
            }
        };

        documents.prototype.editSelectedDoc = function () {
            var grid = this.getDocumentsGrid();
            if (grid) {
                grid.editLastSelectedItem();
            }
        };

        documents.prototype.deleteSelectedDocs = function () {
            var grid = this.getDocumentsGrid();
            if (grid) {
                grid.deleteSelectedItems();
            }
        };

        documents.prototype.copySelectedDocs = function () {
            var grid = this.getDocumentsGrid();
            if (grid) {
                grid.copySelectedDocs();
            }
        };

        documents.prototype.copySelectedDocIds = function () {
            var grid = this.getDocumentsGrid();
            if (grid) {
                grid.copySelectedDocIds();
            }
        };

        documents.prototype.getDocumentsGrid = function () {
            var gridContents = $(documents.gridSelector).children()[0];
            if (gridContents) {
                return ko.dataFor(gridContents);
            }

            return null;
        };
        documents.gridSelector = "#documentsGrid";
        return documents;
    })(viewModelBase);

    
    return documents;
});
