var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "plugins/router", "viewmodels/viewModelBase", "models/index", "models/indexDefinition", "models/indexPriority", "models/luceneField", "models/spatialIndexField", "commands/getIndexDefinitionCommand", "commands/getDatabaseStatsCommand", "commands/saveIndexDefinitionCommand", "common/appUrl", "viewmodels/deleteIndexesConfirm", "plugins/dialog", "common/aceEditorBindingHandler", "common/alertType", "common/alertArgs"], function(require, exports, router, viewModelBase, index, indexDefinition, indexPriority, luceneField, spatialIndexField, getIndexDefinitionCommand, getDatabaseStatsCommand, saveIndexDefinitionCommand, appUrl, deleteIndexesConfirm, dialog, aceEditorBindingHandler, alertType, alertArgs) {
    var editIndex = (function (_super) {
        __extends(editIndex, _super);
        function editIndex() {
            var _this = this;
            _super.call(this);
            this.isEditingExistingIndex = ko.observable(false);
            this.priority = ko.observable().extend({ required: true });
            this.editedIndex = ko.observable();
            this.termsUrl = ko.observable();
            this.statsUrl = ko.observable();
            this.queryUrl = ko.observable();
            this.editMaxIndexOutputsPerDocument = ko.observable(false);
            this.indexErrorsList = ko.observableArray();

            aceEditorBindingHandler.install();

            this.priorityFriendlyName = ko.computed(function () {
                return _this.getPriorityFriendlyName();
            });
            this.priorityLabel = ko.computed(function () {
                return _this.priorityFriendlyName() ? "Priority: " + _this.priorityFriendlyName() : "Priority";
            });
            this.hasExistingReduce = ko.computed(function () {
                return _this.editedIndex() && _this.editedIndex().reduce();
            });
            this.hasExistingTransform = ko.computed(function () {
                return _this.editedIndex() && _this.editedIndex().transformResults();
            });
            this.hasMultipleMaps = ko.computed(function () {
                return _this.editedIndex() && _this.editedIndex().maps().length > 1;
            });
        }
        editIndex.prototype.indexCompleter = function (editor, session, pos, prefix, callback) {
            callback(null, [{ name: "indexa", value: "indexa", score: 10, meta: "trash" }, { name: "indexb", value: "indexb", score: 10, meta: "smashindex" }, { name: "indexc", value: "indexc", score: 10, meta: "indextrash" }]);
        };

        editIndex.prototype.canActivate = function (indexToEditName) {
            var _this = this;
            if (indexToEditName) {
                var canActivateResult = $.Deferred();
                this.fetchIndexToEdit(indexToEditName).done(function () {
                    return canActivateResult.resolve({ can: true });
                }).fail(function () {
                    ko.postbox.publish("Alert", new alertArgs(3 /* danger */, "Could not find " + decodeURIComponent(indexToEditName) + " index", null));
                    canActivateResult.resolve({ redirect: appUrl.forIndexes(_this.activeDatabase()) });
                });

                return canActivateResult;
            } else {
                return $.Deferred().resolve({ can: true });
            }
        };

        editIndex.prototype.activate = function (indexToEditName) {
            _super.prototype.activate.call(this, indexToEditName);

            this.isEditingExistingIndex(indexToEditName != null);

            if (indexToEditName) {
                this.editExistingIndex(indexToEditName);
            } else {
                this.priority(0 /* normal */);
                this.editedIndex(this.createNewIndexDefinition());
            }

            var indexDef = this.editedIndex();
            viewModelBase.dirtyFlag = new ko.DirtyFlag([this.priority, indexDef.name, indexDef.map, indexDef.maps, indexDef.reduce, indexDef.fields, indexDef.transformResults, indexDef.spatialFields, indexDef.maxIndexOutputsPerDocument]);
        };

        editIndex.prototype.attached = function () {
            this.addMapHelpPopover();
            this.addReduceHelpPopover();
            this.addTransformHelpPopover();
        };

        editIndex.prototype.compositionComplete = function () {
            _super.prototype.compositionComplete.call(this);
        };

        editIndex.prototype.editExistingIndex = function (unescapedIndexName) {
            var indexName = decodeURIComponent(unescapedIndexName);
            this.fetchIndexPriority(indexName);
            this.termsUrl(appUrl.forTerms(indexName, this.activeDatabase()));
            this.statsUrl(appUrl.forStatus(this.activeDatabase()));
            this.queryUrl(appUrl.forQuery(this.activeDatabase(), indexName));
        };

        editIndex.prototype.addMapHelpPopover = function () {
            $("#indexMapsLabel").popover({
                html: true,
                trigger: 'hover',
                content: 'Maps project the fields to search on or to group by. It uses LINQ query syntax.<br/><br/>Example:</br><pre><span class="code-keyword">from</span> order <span class="code-keyword">in</span> docs.Orders<br/><span class="code-keyword">where</span> order.IsShipped<br/><span class="code-keyword">select new</span><br/>{</br>   order.Date, <br/>   order.Amount,<br/>   RegionId = order.Region.Id <br />}</pre>Each map function should project the same set of fields.'
            });
        };

        editIndex.prototype.addReduceHelpPopover = function () {
            $("#indexReduceLabel").popover({
                html: true,
                trigger: 'hover',
                content: 'The Reduce function consolidates documents from the Maps stage into a smaller set of documents. It uses LINQ query syntax.<br/><br/>Example:</br><pre><span class="code-keyword">from</span> result <span class="code-keyword">in</span> results<br/><span class="code-keyword">group</span> result <span class="code-keyword">by new</span> { result.RegionId, result.Date }<br/><span class="code-keyword">select new</span><br/>{<br/>  Date = g.Key.Date,<br/>  RegionId = g.Key.RegionId,<br/>  Amount = g.Sum(x => x.Amount)<br/>}</pre>The objects produced by the Reduce function should have the same fields as the inputs.'
            });
        };

        editIndex.prototype.addTransformHelpPopover = function () {
            $("#indexTransformLabel").popover({
                html: true,
                trigger: 'hover',
                content: '<span class="text-danger">Deprecated.</span> Index Transform has been replaced with <strong>Result Transformers</strong>.<br/><br/>The Transform function allows you to change the shape of individual result documents before the server returns them. It uses LINQ query syntax.<br/><br/>Example:<pre><span class="code-keyword">from</span> order <span class="code-keyword">in</span> orders<br/><span class="code-keyword">let</span> region = Database.Load(result.RegionId)<br/><span class="code-keyword">select new</span><br/>{<br/>   result.Date,<br/>   result.Amount,<br/>   Region = region.Name,<br/>   Manager = region.Manager<br/>}</pre>'
            });
        };

        editIndex.prototype.fetchIndexToEdit = function (indexName) {
            var _this = this;
            return new getIndexDefinitionCommand(indexName, this.activeDatabase()).execute().done(function (results) {
                _this.editedIndex(new indexDefinition(results.Index));
                _this.editMaxIndexOutputsPerDocument(results.Index.MaxIndexOutputsPerDocument ? results.Index.MaxIndexOutputsPerDocument > 0 ? true : false : false);
            });
        };

        editIndex.prototype.fetchIndexPriority = function (indexName) {
            var _this = this;
            new getDatabaseStatsCommand(this.activeDatabase()).execute().done(function (stats) {
                var lowerIndexName = indexName.toLowerCase();
                var matchingIndex = stats.Indexes.first(function (i) {
                    return i.PublicName.toLowerCase() === lowerIndexName;
                });
                if (matchingIndex) {
                    var priorityWithoutWhitespace = matchingIndex.Priority.replace(", ", ",");
                    _this.priority(index.priorityFromString(priorityWithoutWhitespace));
                }
            });
        };

        editIndex.prototype.createNewIndexDefinition = function () {
            return indexDefinition.empty();
        };

        editIndex.prototype.save = function () {
            var _this = this;
            if (this.editedIndex().name()) {
                var index = this.editedIndex().toDto();
                var saveCommand = new saveIndexDefinitionCommand(index, this.priority(), this.activeDatabase());
                saveCommand.execute().done(function () {
                    viewModelBase.dirtyFlag().reset();

                    if (!_this.isEditingExistingIndex()) {
                        _this.isEditingExistingIndex(true);
                        _this.editExistingIndex(index.Name);
                    }
                    _this.updateUrl(index.Name);
                });
            }
        };

        editIndex.prototype.updateUrl = function (indexName) {
            router.navigate(appUrl.forEditIndex(indexName, this.activeDatabase()));
        };

        editIndex.prototype.refreshIndex = function () {
            var existingIndex = this.editedIndex();
            if (existingIndex) {
                this.editedIndex(null);
                this.editExistingIndex(existingIndex.name());
            }

            viewModelBase.dirtyFlag().reset();
        };

        editIndex.prototype.deleteIndex = function () {
            var index = this.editedIndex();
            if (index) {
                var db = this.activeDatabase();
                var deleteViewModel = new deleteIndexesConfirm([index.name()], db);
                deleteViewModel.deleteTask.done(function () {
                    return router.navigate(appUrl.forIndexes(db));
                });

                dialog.show(deleteViewModel);
            }
        };

        editIndex.prototype.idlePriority = function () {
            this.priority(4 /* idleForced */);
        };

        editIndex.prototype.disabledPriority = function () {
            this.priority(5 /* disabledForced */);
        };

        editIndex.prototype.abandonedPriority = function () {
            this.priority(6 /* abandonedForced */);
        };

        editIndex.prototype.normalPriority = function () {
            this.priority(0 /* normal */);
        };

        editIndex.prototype.getPriorityFriendlyName = function () {
            var priority = this.priority();
            if (!priority) {
                return "";
            }
            if (priority === 4 /* idleForced */) {
                return index.priorityToString(1 /* idle */);
            }
            if (priority === 5 /* disabledForced */) {
                return index.priorityToString(2 /* disabled */);
            }
            if (priority === 6 /* abandonedForced */) {
                return index.priorityToString(3 /* abandoned */);
            }

            return index.priorityToString(priority);
        };

        editIndex.prototype.addMap = function () {
            this.editedIndex().maps.push(ko.observable());
        };

        editIndex.prototype.addReduce = function () {
            if (!this.hasExistingReduce()) {
                this.editedIndex().reduce(" ");
                this.addReduceHelpPopover();
            }
        };

        editIndex.prototype.addTransform = function () {
            if (!this.hasExistingTransform()) {
                this.editedIndex().transformResults(" ");
                this.addTransformHelpPopover();
            }
        };

        editIndex.prototype.addField = function () {
            var field = new luceneField("");
            this.editedIndex().luceneFields.push(field);
        };

        editIndex.prototype.addSpatialField = function () {
            var field = spatialIndexField.empty();
            this.editedIndex().spatialFields.push(field);
        };

        editIndex.prototype.removeMap = function (mapIndex) {
            this.editedIndex().maps.splice(mapIndex, 1);
        };

        editIndex.prototype.removeReduce = function () {
            this.editedIndex().reduce(null);
        };

        editIndex.prototype.removeTransform = function () {
            this.editedIndex().transformResults(null);
        };

        editIndex.prototype.removeLuceneField = function (fieldIndex) {
            this.editedIndex().luceneFields.splice(fieldIndex, 1);
        };

        editIndex.prototype.removeSpatialField = function (fieldIndex) {
            this.editedIndex().spatialFields.splice(fieldIndex, 1);
        };
        return editIndex;
    })(viewModelBase);

    
    return editIndex;
});
