﻿var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "plugins/router", "common/appUrl", "viewmodels/viewModelBase", "commands/filesystem/getConfigurationCommand", "commands/filesystem/saveConfigurationCommand"], function(require, exports, router, appUrl, viewModelBase, getConfigurationCommand, saveConfigurationCommand) {
    var configuration = (function (_super) {
        __extends(configuration, _super);
        function configuration() {
            var _this = this;
            _super.call(this);
            this.router = router;
            this.keys = ko.observableArray();
            this.selectedKey = ko.observable().subscribeTo("ActivateConfigurationKey").distinctUntilChanged();
            this.keyValues = ko.observableArray();
            this.selectedKeyValue = ko.observable();
            this.currentKey = ko.observable();
            this.selectedKey.subscribe(function (k) {
                return _this.selectedKeyChanged(k);
            });
        }
        configuration.prototype.canActivate = function (args) {
            return true;
        };

        configuration.prototype.attached = function () {
            var _this = this;
            this.activeFilesystem.subscribe(function (x) {
                _this.loadKeys(x);
            });

            $('.keys-collection').contextmenu({
                target: '#keys-context-menu'
            });

            this.loadKeys(this.activeFilesystem());
        };

        configuration.prototype.loadKeys = function (fs) {
            var _this = this;
            new getConfigurationCommand(fs).execute().done(function (x) {
                _this.keys(x);
            });
        };

        configuration.prototype.selectKey = function (key) {
            key.activate();
            router.navigate(appUrl.forFilesystemConfigurationWithKey(this.activeFilesystem(), key.key));
        };

        configuration.prototype.selectedKeyChanged = function (selected) {
            var _this = this;
            if (selected) {
                selected.getValues().done(function (x) {
                    var nameValueCollection = new Array();

                    for (var i = 0; i < x.length; i++) {
                        var pair = x[i];
                        var name = pair.item1;
                        for (var j = 0; j < pair.item2.length; j++) {
                            var value = pair.item2[j];
                            nameValueCollection.push(new Pair(name, value));
                        }
                    }

                    if (nameValueCollection.length == 0)
                        nameValueCollection.push(new Pair("", ""));

                    _this.keyValues(nameValueCollection);
                });

                this.currentKey(selected);
            }
        };

        configuration.prototype.selectKeyValue = function (selection) {
            this.selectedKeyValue(selection);
        };

        configuration.prototype.save = function () {
            var args = {};
            var values = this.keyValues();

            for (var i = 0; i < values.length; i++) {
                var key = values[i].item1;
                if (key == null || key == "")
                    continue;

                var value = values[i].item2;
                if (value == null || value == "")
                    continue;

                if (args[key] == null) {
                    args[key] = [value];
                } else {
                    args[key].push(value);
                }
            }

            new saveConfigurationCommand(this.activeFilesystem(), this.currentKey(), args).execute();
        };

        configuration.prototype.deleteKey = function () {
            throw new Error("Not Implemented");
        };

        configuration.prototype.addKeyValue = function () {
            this.keyValues.push(new Pair("", ""));
        };

        configuration.prototype.removeKeyValue = function (keyValue) {
            if (keyValue) {
                this.keyValues.remove(keyValue);
            }

            if (this.keyValues.length == 0) {
                this.keyValues.push(new Pair("", ""));
            }
        };
        return configuration;
    })(viewModelBase);

    
    return configuration;
});
