﻿var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "plugins/router", "common/appUrl", "common/pagedList", "commands/filesystem/getFilesCommand", "viewmodels/viewModelBase"], function(require, exports, router, appUrl, pagedList, getFilesystemFilesCommand, viewModelBase) {
    var filesystemFiles = (function (_super) {
        __extends(filesystemFiles, _super);
        function filesystemFiles() {
            _super.apply(this, arguments);
            this.fileName = ko.observable();
            this.allFilesPagedItems = ko.observable();
            this.selectedFilesIndices = ko.observableArray();
            this.isSelectAll = ko.observable(false);
        }
        filesystemFiles.prototype.canActivate = function (args) {
            return true;
        };

        filesystemFiles.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);
            this.activeFilesystem.subscribe(function (fs) {
                return _this.fileSystemChanged(fs);
            });
            this.hasAnyFileSelected = ko.computed(function () {
                return _this.selectedFilesIndices().length > 0;
            });

            this.loadFiles(false);
        };

        filesystemFiles.prototype.loadFiles = function (force) {
            if (!this.allFilesPagedItems() || force) {
                this.allFilesPagedItems(this.createPagedList());
            }

            return this.allFilesPagedItems;
        };

        filesystemFiles.prototype.createPagedList = function () {
            var _this = this;
            var fetcher = function (skip, take) {
                return _this.fetchFiles(skip, take);
            };
            var list = new pagedList(fetcher);
            return list;
        };

        filesystemFiles.prototype.fetchFiles = function (skip, take) {
            var task = new getFilesystemFilesCommand(appUrl.getFilesystem(), skip, take).execute();

            return task;
        };

        filesystemFiles.prototype.fileSystemChanged = function (fs) {
            if (fs) {
                var filesystemFilesUrl = appUrl.forFilesystemFiles(fs);
                this.navigate(filesystemFilesUrl);

                this.loadFiles(true);
            }
        };

        filesystemFiles.prototype.editSelectedFile = function () {
            var grid = this.getFilesGrid();
            if (grid) {
                grid.editLastSelectedItem();
            }
        };

        filesystemFiles.prototype.toggleSelectAll = function () {
            this.isSelectAll.toggle();

            var filesGrid = this.getFilesGrid();
            if (filesGrid && this.isSelectAll()) {
                filesGrid.selectAll();
            } else if (filesGrid && !this.isSelectAll()) {
                filesGrid.selectNone();
            }
        };

        filesystemFiles.prototype.getFilesGrid = function () {
            var gridContents = $(filesystemFiles.gridSelector).children()[0];
            if (gridContents) {
                return ko.dataFor(gridContents);
            }

            return null;
        };

        filesystemFiles.prototype.deleteSelectedFiles = function () {
            var grid = this.getFilesGrid();
            if (grid) {
                grid.deleteSelectedItems();
            }
        };

        filesystemFiles.prototype.downloadSelectedFiles = function () {
            var grid = this.getFilesGrid();
            if (grid) {
                var selectedItem = grid.getSelectedItems(1).first();
                var url = appUrl.forResourceQuery(this.activeFilesystem()) + "/files/" + selectedItem.getId();
                window.location.assign(url);
            }
        };

        filesystemFiles.prototype.uploadFile = function () {
            router.navigate(appUrl.forFilesystemUploadFile(this.activeFilesystem()));
        };
        filesystemFiles.gridSelector = "#filesGrid";
        return filesystemFiles;
    })(viewModelBase);

    
    return filesystemFiles;
});
