var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "durandal/app", "plugins/router", "common/appUrl", "models/filesystem/filesystem", "commands/filesystem/getFilesystemsCommand", "commands/filesystem/getFilesystemStatsCommand", "viewmodels/viewModelBase", "commands/filesystem/createFilesystemCommand"], function(require, exports, app, router, appUrl, filesystem, getFilesystemsCommand, getFilesystemStatsCommand, viewModelBase, createFilesystemCommand) {
    var filesystems = (function (_super) {
        __extends(filesystems, _super);
        function filesystems() {
            var _this = this;
            _super.call(this);
            this.filesystems = ko.observableArray();
            this.hasFilesystems = ko.computed(function () {
                return _this.filesystems().length > 0;
            });
            this.searchText = ko.observable("");
            this.selectedFilesystem = ko.observable();

            this.defaultFs = appUrl.getDefaultFilesystem();
            this.searchText.extend({ throttle: 200 }).subscribe(function (s) {
                return _this.filterFilesystems(s);
            });
        }
        filesystems.prototype.canActivate = function (args) {
            return true;
        };

        filesystems.prototype.modelPolling = function () {
            var _this = this;
            new getFilesystemsCommand().execute().done(function (results) {
                return _this.filesystemsLoaded(results);
            });
        };

        filesystems.prototype.filterFilesystems = function (filter) {
            var filterLower = filter.toLowerCase();
            this.filesystems().forEach(function (d) {
                var isMatch = !filter || (d.name.toLowerCase().indexOf(filterLower) >= 0);
                d.isVisible(isMatch);
            });
        };

        filesystems.prototype.navigateToFiles = function (fs) {
            fs.activate();
            router.navigate(appUrl.forFilesystemFiles(fs));
        };

        filesystems.prototype.getFilesystemFilesUrl = function (fs) {
            return appUrl.forFilesystemFiles(fs);
        };

        filesystems.prototype.filesystemsLoaded = function (results) {
            var _this = this;
            var filesystemsHaveChanged = this.checkDifferentFilesystems(results);
            if (filesystemsHaveChanged) {
                this.filesystems(results);

                var few = 20;
                if (results.length < few && !this.initializedStats) {
                    this.initializedStats = true;
                    results.forEach(function (fs) {
                        return _this.fetchStats(fs);
                    });
                }
            }
        };

        filesystems.prototype.checkDifferentFilesystems = function (fss) {
            if (fss.length !== this.filesystems().length) {
                return true;
            }

            var freshFsNames = fss.map(function (fs) {
                return fs.name;
            });
            var existingFsNames = this.filesystems().map(function (fs) {
                return fs.name;
            });
            return existingFsNames.some(function (existing) {
                return !freshFsNames.contains(existing);
            });
        };

        filesystems.prototype.fetchStats = function (fs) {
            new getFilesystemStatsCommand(fs).execute().done(function (result) {
                return fs.statistics(result);
            });
        };

        filesystems.prototype.selectFilesystem = function (fs) {
            this.filesystems().forEach(function (d) {
                return d.isSelected(d === fs);
            });
            fs.activate();
            this.selectedFilesystem(fs);
        };

        filesystems.prototype.newFilesystem = function () {
            var _this = this;
            require(["viewmodels/filesystem/createFilesystem"], function (createFilesystem) {
                var createFilesystemViewModel = new createFilesystem(_this.filesystems);
                createFilesystemViewModel.creationTask.done(function (filesystemName, filesystemPath) {
                    return _this.showCreationAdvancedStepsIfNecessary(filesystemName, filesystemPath);
                });
                app.showDialog(createFilesystemViewModel);
            });
        };

        filesystems.prototype.showCreationAdvancedStepsIfNecessary = function (filesystemName, filesystemPath) {
            var _this = this;
            new createFilesystemCommand(filesystemName, filesystemPath).execute().done(function () {
                _this.filesystems.unshift(new filesystem(filesystemName));
            });
        };
        return filesystems;
    })(viewModelBase);

    
    return filesystems;
});
