var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "viewmodels/viewModelBase", "models/patchDocuments", "common/aceEditorBindingHandler", "commands/getDatabaseStatsCommand", "commands/getCollectionsCommand", "common/pagedList", "commands/queryIndexCommand", "commands/getDocumentWithMetadataCommand"], function(require, exports, viewModelBase, patchDocuments, aceEditorBindingHandler, getDatabaseStatsCommand, getCollectionsCommand, pagedList, queryIndexCommand, getDocumentWithMetadataCommand) {
    var patch = (function (_super) {
        __extends(patch, _super);
        function patch() {
            _super.call(this);
            this.displayName = "patch";
            this.indexNames = ko.observableArray();
            this.collections = ko.observableArray();
            this.currentCollectionPagedItems = ko.observable();
            this.selectedDocumentIndices = ko.observableArray();
            this.patchDocuments = ko.observable();
            this.beforePatch = ko.observable();
            this.afterPatch = ko.observable();

            aceEditorBindingHandler.install();
        }
        patch.prototype.activate = function () {
            var self = this;
            this.patchDocuments(patchDocuments.empty());
        };

        patch.prototype.loadDocumentToPatch = function (selectedItem) {
            var _this = this;
            var loadDocTask = new getDocumentWithMetadataCommand(selectedItem, this.activeDatabase()).execute();
            loadDocTask.done(function (document) {
                _this.beforePatch(JSON.stringify(document.toDto(), null, 4));
            }).fail(this.clearDocumentPreview());
        };

        patch.prototype.clearDocumentPreview = function () {
            this.beforePatch('');
            this.afterPatch('');
        };

        patch.prototype.setSelectedPatchOnOption = function (patchOnOption) {
            this.patchDocuments().patchOnOption(patchOnOption);
            this.patchDocuments().selectedItem('');
            this.clearDocumentPreview();
            switch (patchOnOption) {
                case "Collection":
                    this.fetchAllCollections();
                    break;
                case "Index":
                    this.fetchAllIndexes();
                    break;
            }
        };

        patch.prototype.fetchAllCollections = function () {
            var _this = this;
            return new getCollectionsCommand(this.activeDatabase()).execute().done(function (colls) {
                _this.collections(colls);
                if (_this.collections().length > 0) {
                    _this.setSelectedCollection(_this.collections().first());
                }
            });
        };

        patch.prototype.setSelectedCollection = function (coll) {
            this.patchDocuments().selectedItem(coll.name);
            this.currentCollectionPagedItems(coll.getDocuments());
        };

        patch.prototype.fetchAllIndexes = function () {
            var _this = this;
            return new getDatabaseStatsCommand(this.activeDatabase()).execute().done(function (results) {
                _this.indexNames(results.Indexes.map(function (i) {
                    return i.PublicName;
                }));
                if (_this.indexNames().length > 0) {
                    _this.setSelectedIndex(_this.indexNames().first());
                }
            });
        };

        patch.prototype.setSelectedIndex = function (indexName) {
            this.patchDocuments().selectedItem(indexName);
            this.runQuery();
        };

        patch.prototype.runQuery = function () {
            var selectedIndex = this.patchDocuments().selectedItem();
            if (selectedIndex) {
                var queryText = this.patchDocuments().query();
                var database = this.activeDatabase();
                var resultsFetcher = function (skip, take) {
                    var command = new queryIndexCommand(selectedIndex, database, skip, take, queryText, []);
                    return command.execute();
                };
                var resultsList = new pagedList(resultsFetcher);
                this.currentCollectionPagedItems(resultsList);
                return resultsList;
            }

            return null;
        };
        return patch;
    })(viewModelBase);

    
    return patch;
});
