var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "durandal/app", "common/appUrl", "viewmodels/viewModelBase", "commands/getDatabaseStatsCommand", "commands/getCollectionsCommand", "commands/getIndexDefinitionCommand", "common/aceEditorBindingHandler", "common/pagedList", "commands/queryIndexCommand", "models/querySort", "models/collection", "commands/getTransformersCommand", "viewmodels/deleteDocumentsMatchingQueryConfirm", "commands/getStoredQueriesCommand", "commands/saveDocumentCommand", "models/document", 'models/customColumnParams', 'models/customColumns', 'viewmodels/selectColumns', 'commands/getCustomColumnsCommand', "commands/getDocumentsByEntityNameCommand", "commands/getDocumentsMetadataByIDPrefixCommand", "commands/getIndexTermsCommand"], function(require, exports, app, appUrl, viewModelBase, getDatabaseStatsCommand, getCollectionsCommand, getIndexDefinitionCommand, aceEditorBindingHandler, pagedList, queryIndexCommand, querySort, collection, getTransformersCommand, deleteDocumentsMatchingQueryConfirm, getStoredQueriesCommand, saveDocumentCommand, document, customColumnParams, customColumns, selectColumns, getCustomColumnsCommand, getDocumentsByEntityNameCommand, getDocumentsMetadataByIDPrefixCommand, getIndexTermsCommand) {
    var query = (function (_super) {
        __extends(query, _super);
        function query() {
            var _this = this;
            _super.call(this);
            this.selectedIndex = ko.observable();
            this.indexes = ko.observableArray();
            this.queryText = ko.observable("");
            this.queryResults = ko.observable();
            this.selectedResultIndices = ko.observableArray();
            this.queryStats = ko.observable();
            this.sortBys = ko.observableArray();
            this.indexFields = ko.observableArray();
            this.transformer = ko.observable();
            this.allTransformers = ko.observableArray();
            this.isDefaultOperatorOr = ko.observable(true);
            this.showFields = ko.observable(false);
            this.indexEntries = ko.observable(false);
            this.recentQueries = ko.observableArray();
            this.recentQueriesDoc = ko.observable();
            this.rawJsonUrl = ko.observable();
            this.collectionNames = ko.observableArray();
            this.isIndexMapReduce = ko.computed(function () {
                var currentIndex = _this.indexes.first(function (i) {
                    return i.name == _this.selectedIndex();
                });
                return !!currentIndex && currentIndex.hasReduce == true;
            });
            this.contextName = ko.observable();
            this.currentColumnsParams = ko.observable(customColumns.empty());
            this.appUrls = appUrl.forCurrentDatabase();
            this.editIndexUrl = ko.computed(function () {
                return _this.selectedIndex() ? appUrl.forEditIndex(_this.selectedIndex(), _this.activeDatabase()) : null;
            });
            this.termsUrl = ko.computed(function () {
                return _this.selectedIndex() ? appUrl.forTerms(_this.selectedIndex(), _this.activeDatabase()) : null;
            });
            this.statsUrl = ko.computed(function () {
                return appUrl.forStatus(_this.activeDatabase());
            });
            this.hasSelectedIndex = ko.computed(function () {
                return _this.selectedIndex() != null;
            });
            this.rawJsonUrl.subscribe(function (value) {
                return ko.postbox.publish("SetRawJSONUrl", value);
            });
            this.selectedIndexLabel = ko.computed(function () {
                return _this.selectedIndex() === "dynamic" ? "All Documents" : _this.selectedIndex();
            });
            this.selectedIndexEditUrl = ko.computed(function () {
                if (_this.queryStats()) {
                    var index = _this.queryStats().IndexName;
                    if (index && index.indexOf("dynamic/") !== 0) {
                        return appUrl.forEditIndex(index, _this.activeDatabase());
                    }
                }

                return "";
            });

            this.didDynamicChangeIndex = ko.computed(function () {
                if (_this.queryStats()) {
                    var recievedIndex = _this.queryStats().IndexName;
                    var selectedIndex = _this.selectedIndex();
                    return selectedIndex.indexOf("dynamic/") === 0 && _this.indexes()[0].name !== recievedIndex;
                } else {
                    return false;
                }
            });

            aceEditorBindingHandler.install();
        }
        query.prototype.activate = function (indexNameOrRecentQueryHash) {
            var _this = this;
            _super.prototype.activate.call(this, indexNameOrRecentQueryHash);

            this.fetchAllTransformers();
            $.when(this.fetchAllCollections(), this.fetchAllIndexes(), this.fetchRecentQueries()).done(function () {
                return _this.selectInitialQuery(indexNameOrRecentQueryHash);
            });

            this.selectedIndex.subscribe(function (index) {
                return _this.onIndexChanged(index);
            });
        };

        query.prototype.attached = function () {
            var _this = this;
            _super.prototype.attached.call(this);
            this.createKeyboardShortcut("F2", function () {
                return _this.editSelectedIndex();
            }, query.containerSelector);
            this.createKeyboardShortcut("ctrl+enter", function () {
                return _this.runQuery();
            }, query.containerSelector);
            this.createKeyboardShortcut("alt+c", function () {
                return _this.focusOnQuery();
            }, query.containerSelector);
            $("#indexQueryLabel").popover({
                html: true,
                trigger: 'hover',
                container: '.form-horizontal',
                content: 'Queries use Lucene syntax. Examples:<pre><span class="code-keyword">Name</span>: Hi?berna*<br/><span class="code-keyword">Count</span>: [0 TO 10]<br/><span class="code-keyword">Title</span>: "RavenDb Queries 1010" AND <span class="code-keyword">Price</span>: [10.99 TO *]</pre>'
            });
            ko.postbox.publish("SetRawJSONUrl", appUrl.forIndexQueryRawData(this.activeDatabase(), this.selectedIndex()));

            _super.prototype.createResizableTextBoxes.call(this);
            this.focusOnQuery();
        };

        query.prototype.onIndexChanged = function (newIndexName) {
            var _this = this;
            var command = getCustomColumnsCommand.forIndex(newIndexName, this.activeDatabase());
            this.contextName(command.docName);

            command.execute().done(function (dto) {
                if (dto) {
                    _this.currentColumnsParams().columns($.map(dto.Columns, function (c) {
                        return new customColumnParams(c);
                    }));
                    _this.currentColumnsParams().customMode(true);
                } else {
                    _this.currentColumnsParams().columns.removeAll();
                    _this.currentColumnsParams().customMode(false);
                }
            });
        };

        query.prototype.selectInitialQuery = function (indexNameOrRecentQueryHash) {
            if (!indexNameOrRecentQueryHash && this.indexes().length > 0) {
                this.setSelectedIndex(this.indexes.first().name);
            } else if (this.indexes.first(function (i) {
                return i.name == indexNameOrRecentQueryHash;
            }) || indexNameOrRecentQueryHash.indexOf("dynamic/") === 0 || indexNameOrRecentQueryHash === "dynamic") {
                this.setSelectedIndex(indexNameOrRecentQueryHash);
            } else if (indexNameOrRecentQueryHash.indexOf("recentquery-") === 0) {
                var hash = parseInt(indexNameOrRecentQueryHash.substr("recentquery-".length), 10);
                var matchingQuery = this.recentQueries.first(function (q) {
                    return q.Hash === hash;
                });
                if (matchingQuery) {
                    this.runRecentQuery(matchingQuery);
                }
            }
        };

        query.prototype.focusOnQuery = function () {
            var editorElement = $("#queryEditor").length == 1 ? $("#queryEditor")[0] : null;
            if (editorElement) {
                var docEditor = ko.utils.domData.get($("#queryEditor")[0], "aceEditor");
                if (docEditor) {
                    docEditor.focus();
                }
            }
        };

        query.prototype.editSelectedIndex = function () {
            this.navigate(this.editIndexUrl());
        };

        query.prototype.fetchAllIndexes = function () {
            var _this = this;
            return new getDatabaseStatsCommand(this.activeDatabase()).execute().done(function (results) {
                return _this.indexes(results.Indexes.map(function (i) {
                    return {
                        name: i.PublicName,
                        hasReduce: !!i.LastReducedTimestamp
                    };
                }));
            });
        };

        query.prototype.fetchAllCollections = function () {
            var _this = this;
            return new getCollectionsCommand(this.activeDatabase()).execute().done(function (results) {
                return _this.collectionNames(results.map(function (c) {
                    return c.name;
                }));
            });
        };

        query.prototype.fetchRecentQueries = function () {
            var _this = this;
            var result = $.Deferred();
            var storedQueriesCommand = new getStoredQueriesCommand(this.activeDatabase());
            storedQueriesCommand.execute().fail(function (_) {
                var newStoredQueryContainer = {
                    '@metadata': {},
                    Queries: []
                };
                _this.recentQueriesDoc(newStoredQueryContainer);
                _this.recentQueries(newStoredQueryContainer.Queries);
            }).done(function (doc) {
                var dto = doc.toDto(true);
                _this.recentQueriesDoc(dto);
                _this.recentQueries(dto.Queries);
            }).always(function () {
                return result.resolve();
            });

            return result;
        };

        query.prototype.fetchAllTransformers = function () {
            var _this = this;
            new getTransformersCommand(this.activeDatabase()).execute().done(function (results) {
                return _this.allTransformers(results);
            });
        };

        query.prototype.runRecentQuery = function (query) {
            this.selectedIndex(query.IndexName);
            this.queryText(query.QueryText);
            this.showFields(query.ShowFields);
            this.indexEntries(query.IndexEntries);
            this.isDefaultOperatorOr(query.UseAndOperator === false);
            this.transformer(query.TransformerName);
            this.sortBys(query.Sorts.map(function (s) {
                return querySort.fromQuerySortString(s);
            }));
            this.runQuery();
        };

        query.prototype.runQuery = function () {
            var _this = this;
            var selectedIndex = this.selectedIndex();
            if (selectedIndex) {
                var queryText = this.queryText();
                var sorts = this.sortBys().filter(function (s) {
                    return s.fieldName() != null;
                });
                var database = this.activeDatabase();
                var transformer = this.transformer();
                var showFields = this.showFields();
                var indexEntries = this.indexEntries();

                this.currentColumnsParams().enabled(this.showFields() === false && this.indexEntries() === false);

                var useAndOperator = this.isDefaultOperatorOr() === false;
                this.rawJsonUrl(appUrl.forResourceQuery(this.activeDatabase()) + new queryIndexCommand(selectedIndex, database, 0, 1024, queryText, sorts, transformer, showFields, indexEntries, useAndOperator).getUrl());
                var resultsFetcher = function (skip, take) {
                    var command = new queryIndexCommand(selectedIndex, database, skip, take, queryText, sorts, transformer, showFields, indexEntries, useAndOperator);
                    return command.execute().done(function (queryResults) {
                        return _this.queryStats(queryResults.additionalResultInfo);
                    });
                };
                var resultsList = new pagedList(resultsFetcher);
                this.queryResults(resultsList);
                this.recordQueryRun(selectedIndex, queryText, sorts.map(function (s) {
                    return s.toQuerySortString();
                }), transformer, showFields, indexEntries, useAndOperator);

                return resultsList;
            }

            return null;
        };

        query.prototype.queryCompleter = function (editor, session, pos, prefix, callback) {
            var currentToken = session.getTokenAt(pos.row, pos.column);

            if (!currentToken || typeof currentToken.type == "string") {
                if (!currentToken || currentToken.type == "text") {
                    callback(null, this.indexFields().map(function (curColumn) {
                        return { name: curColumn, value: curColumn, score: 10, meta: "field" };
                    }));
                } else if (currentToken.type == "keyword" || currentToken.type == "value") {
                    var currentColumnName = null;
                    var currentValue = "";

                    if (currentToken.type == "keyword") {
                        currentColumnName = currentToken.value.substring(0, currentToken.value.length - 1);
                    } else {
                        currentValue = currentToken.value.trim();
                        var rowTokens = session.getTokens(pos.row);
                        if (!!rowTokens && rowTokens.length > 1) {
                            currentColumnName = rowTokens[rowTokens.length - 2].value.trim();
                            currentColumnName = currentColumnName.substring(0, currentColumnName.length - 1);
                        }
                    }

                    if (!!currentColumnName && !!this.indexFields.first(function (x) {
                        return x === currentColumnName;
                    })) {
                        if (this.selectedIndex().indexOf("dynamic/") !== 0) {
                            new getIndexTermsCommand(this.selectedIndex(), currentColumnName, this.activeDatabase()).execute().done(function (terms) {
                                if (!!terms && terms.length > 0) {
                                    callback(null, terms.map(function (curVal) {
                                        return { name: curVal, value: curVal, score: 10, meta: "value" };
                                    }));
                                }
                            });
                        } else {
                            if (currentValue.length > 0) {
                                new getDocumentsMetadataByIDPrefixCommand(currentValue, 10, this.activeDatabase()).execute().done(function (results) {
                                    if (!!results && results.length > 0) {
                                        callback(null, results.map(function (curVal) {
                                            return { name: curVal['@metadata']['@id'], value: curVal['@metadata']['@id'], score: 10, meta: "value" };
                                        }));
                                    }
                                });
                            } else {
                                callback([{ error: "notext" }], null);
                            }
                        }
                    }
                }
            }
        };

        query.prototype.recordQueryRun = function (indexName, queryText, sorts, transformer, showFields, indexEntries, useAndOperator) {
            var newQuery = {
                IndexEntries: indexEntries,
                IndexName: indexName,
                IsPinned: false,
                QueryText: queryText,
                ShowFields: showFields,
                Sorts: sorts,
                TransformerName: transformer || null,
                UseAndOperator: useAndOperator,
                Hash: (indexName + (queryText || "") + sorts.reduce(function (a, b) {
                    return a + b;
                }, "") + (transformer || "") + showFields + indexEntries + useAndOperator).hashCode()
            };

            var queryUrl = appUrl.forQuery(this.activeDatabase(), newQuery.Hash);
            this.updateUrl(queryUrl);

            var existing = this.recentQueries.first(function (q) {
                return q.Hash === newQuery.Hash;
            });
            if (existing) {
                this.recentQueries.remove(existing);
                this.recentQueries.unshift(existing);
            } else {
                this.recentQueries.unshift(newQuery);
            }

            if (this.recentQueries().length > 15) {
                this.recentQueries.remove(this.recentQueries()[15]);
            }

            var recentQueriesDoc = this.recentQueriesDoc();
            if (recentQueriesDoc) {
                recentQueriesDoc.Queries = this.recentQueries();
                var preppedDoc = new document(recentQueriesDoc);
                new saveDocumentCommand(getStoredQueriesCommand.storedQueryDocId, preppedDoc, this.activeDatabase(), false).execute().done(function (result) {
                    return recentQueriesDoc['@metadata']['@etag'] = result.ETag;
                });
            }
        };

        query.prototype.getRecentQuerySortsText = function (recentQueryIndex) {
            var sorts = this.recentQueries()[recentQueryIndex].Sorts;
            if (sorts.length === 0) {
                return "";
            }
            return sorts.map(function (s) {
                return querySort.fromQuerySortString(s);
            }).map(function (s) {
                return s.toHumanizedString();
            }).reduce(function (first, second) {
                return first + ", " + second;
            });
        };

        query.prototype.setSelectedIndex = function (indexName) {
            var _this = this;
            this.sortBys.removeAll();
            this.selectedIndex(indexName);
            this.runQuery();

            var indexQuery = query.getIndexUrlPartFromIndexName(indexName);
            var url = appUrl.forQuery(this.activeDatabase(), indexQuery);
            this.updateUrl(url);
            NProgress.done();

            var isAllDocumentsDynamicQuery = indexName === "All Documents";
            if (!isAllDocumentsDynamicQuery) {
                if (indexName.indexOf('dynamic/') == 0) {
                    var collectionName = indexName.substring(8);
                    new getDocumentsByEntityNameCommand(new collection(collectionName, this.activeDatabase()), 0, 1).execute().done(function (result) {
                        if (!!result && result.totalResultCount > 0) {
                            var dynamicIndexPattern = new document(result.items[0]);
                            if (!!dynamicIndexPattern) {
                                _this.indexFields(dynamicIndexPattern.getDocumentPropertyNames());
                            }
                        }
                    });
                } else {
                    new getIndexDefinitionCommand(indexQuery, this.activeDatabase()).execute().done(function (result) {
                        _this.indexFields(result.Index.Fields);
                    });
                }
            }
        };

        query.getIndexUrlPartFromIndexName = function (indexNameOrCollectionName) {
            if (indexNameOrCollectionName === "All Documents") {
                return "dynamic";
            }

            return indexNameOrCollectionName;
        };

        query.prototype.addSortBy = function () {
            var _this = this;
            var sort = new querySort();
            sort.fieldName.subscribe(function () {
                return _this.runQuery();
            });
            sort.sortDirection.subscribe(function () {
                return _this.runQuery();
            });
            this.sortBys.push(sort);
        };

        query.prototype.removeSortBy = function (sortBy) {
            this.sortBys.remove(sortBy);
            this.runQuery();
        };

        query.prototype.addTransformer = function () {
            this.transformer("");
        };

        query.prototype.selectTransformer = function (transformer) {
            this.transformer(transformer.name);
            this.runQuery();
        };

        query.prototype.removeTransformer = function () {
            this.transformer(null);
            this.runQuery();
        };

        query.prototype.setOperatorOr = function () {
            this.isDefaultOperatorOr(true);
            this.runQuery();
        };

        query.prototype.setOperatorAnd = function () {
            this.isDefaultOperatorOr(false);
            this.runQuery();
        };

        query.prototype.toggleShowFields = function () {
            this.showFields(!this.showFields());
            this.runQuery();
        };

        query.prototype.toggleIndexEntries = function () {
            this.indexEntries(!this.indexEntries());
            this.runQuery();
        };

        query.prototype.deleteDocsMatchingQuery = function () {
            var _this = this;
            var queryResult = this.runQuery();
            queryResult.fetch(0, 1).done(function (results) {
                if (results.totalResultCount === 0) {
                    app.showMessage("There are no documents matching your query.", "Nothing to do");
                } else {
                    _this.promptDeleteDocsMatchingQuery(results.totalResultCount);
                }
            });
        };

        query.prototype.promptDeleteDocsMatchingQuery = function (resultCount) {
            var _this = this;
            var viewModel = new deleteDocumentsMatchingQueryConfirm(this.selectedIndex(), this.queryText(), resultCount, this.activeDatabase());
            app.showDialog(viewModel).done(function () {
                return _this.runQuery();
            });
        };

        query.prototype.selectColumns = function () {
            var _this = this;
            var selectColumnsViewModel = new selectColumns(this.currentColumnsParams().clone(), this.contextName(), this.activeDatabase());
            app.showDialog(selectColumnsViewModel);
            selectColumnsViewModel.onExit().done(function (cols) {
                _this.currentColumnsParams(cols);

                _this.runQuery();
            });
        };
        query.containerSelector = "#queryContainer";
        return query;
    })(viewModelBase);

    
    return query;
});
