var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "viewmodels/viewModelBase", "commands/getReplicationStatsCommand", "moment"], function(require, exports, viewModelBase, getReplicationStatsCommand, moment) {
    var replicationStats = (function (_super) {
        __extends(replicationStats, _super);
        function replicationStats() {
            _super.call(this);
            this.replStatsDoc = ko.observable();
            this.hasNoReplStatsAvailable = ko.observable(false);
            this.now = ko.observable();
            this.updateNowTimeoutHandle = 0;

            this.updateCurrentNowTime();
        }
        replicationStats.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);

            this.activeDatabase.subscribe(function () {
                return _this.fetchReplStats();
            });
            this.fetchReplStats();
        };

        replicationStats.prototype.fetchReplStats = function () {
            var _this = this;
            this.replStatsDoc(null);
            new getReplicationStatsCommand(this.activeDatabase()).execute().fail(function () {
                return _this.hasNoReplStatsAvailable(true);
            }).done(function (result) {
                _this.hasNoReplStatsAvailable(result.Stats.length === 0);
                _this.processResults(result);
            });
        };

        replicationStats.prototype.processResults = function (results) {
            var _this = this;
            if (results) {
                results.Stats.forEach(function (s) {
                    s['LastReplicatedLastModifiedHumanized'] = _this.createHumanReadableTime(s.LastReplicatedLastModified);
                    s['LastFailureTimestampHumanized'] = _this.createHumanReadableTime(s.LastFailureTimestamp);
                    s['LastHeartbeatReceivedHumanized'] = _this.createHumanReadableTime(s.LastHeartbeatReceived);
                    s['LastSuccessTimestampHumanized'] = _this.createHumanReadableTime(s.LastSuccessTimestamp);
                });
            }

            this.replStatsDoc(results);
        };

        replicationStats.prototype.createHumanReadableTime = function (time) {
            var _this = this;
            if (time) {
                return ko.computed(function () {
                    var dateMoment = moment(time);
                    var agoInMs = dateMoment.diff(_this.now());
                    return moment.duration(agoInMs).humanize(true) + dateMoment.format(" (MM/DD/YY, h:mma)");
                });
            }

            return ko.computed(function () {
                return time;
            });
        };

        replicationStats.prototype.updateCurrentNowTime = function () {
            var _this = this;
            this.now(moment());
            this.updateNowTimeoutHandle = setTimeout(function () {
                return _this.updateCurrentNowTime();
            }, 60000);
        };
        return replicationStats;
    })(viewModelBase);

    
    return replicationStats;
});
