var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "viewmodels/viewModelBase", "models/replicationsSetup", "models/replicationConfig", "models/replicationDestination", "commands/getDatabaseStatsCommand", "commands/getReplicationsCommand", "commands/saveReplicationDocumentCommand", "commands/getAutomaticConflictResolutionDocumentCommand", "commands/saveAutomaticConflictResolutionDocument", "common/appUrl"], function(require, exports, viewModelBase, replicationsSetup, replicationConfig, replicationDestination, getDatabaseStatsCommand, getReplicationsCommand, saveReplicationDocumentCommand, getAutomaticConflictResolutionDocumentCommand, saveAutomaticConflictResolutionDocument, appUrl) {
    var replications = (function (_super) {
        __extends(replications, _super);
        function replications() {
            _super.apply(this, arguments);
            this.replicationConfig = ko.observable(new replicationConfig({ DocumentConflictResolution: "None", AttachmentConflictResolution: "None" }));
            this.replicationsSetup = ko.observable(new replicationsSetup({ Destinations: [], Source: null }));
            this.replicationConfigDirtyFlag = new ko.DirtyFlag([]);
            this.replicationsSetupDirtyFlag = new ko.DirtyFlag([]);
        }
        replications.prototype.canActivate = function (args) {
            var _this = this;
            var deferred = $.Deferred();
            var db = this.activeDatabase();
            if (db) {
                $.when(this.fetchAutomaticConflictResolution(db), this.fetchReplications(db)).done(function () {
                    return deferred.resolve({ can: true });
                }).fail(function () {
                    return deferred.resolve({ redirect: appUrl.forIndexes(_this.activeDatabase()) });
                });
            }
            return deferred;
        };

        replications.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);

            var self = this;
            this.replicationConfigDirtyFlag = new ko.DirtyFlag([this.replicationConfig]);
            this.isConfigSaveEnabled = ko.computed(function () {
                return self.replicationConfigDirtyFlag().isDirty();
            });
            this.replicationsSetupDirtyFlag = new ko.DirtyFlag([this.replicationsSetup, this.replicationsSetup().destinations()]);
            this.isSetupSaveEnabled = ko.computed(function () {
                return self.replicationsSetupDirtyFlag().isDirty();
            });

            var combinedFlag = ko.computed(function () {
                return (self.replicationConfigDirtyFlag().isDirty() || self.replicationsSetupDirtyFlag().isDirty());
            });
            viewModelBase.dirtyFlag = new ko.DirtyFlag([combinedFlag]);
        };

        replications.prototype.fetchAutomaticConflictResolution = function (db) {
            var _this = this;
            var deferred = $.Deferred();
            new getAutomaticConflictResolutionDocumentCommand(db).execute().done(function (repConfig) {
                return _this.replicationConfig(new replicationConfig(repConfig));
            }).always(function () {
                return deferred.resolve({ can: true });
            });
            return deferred;
        };

        replications.prototype.fetchReplications = function (db) {
            var _this = this;
            var deferred = $.Deferred();
            new getReplicationsCommand(db).execute().done(function (repSetup) {
                return _this.replicationsSetup(new replicationsSetup(repSetup));
            }).always(function () {
                return deferred.resolve({ can: true });
            });
            return deferred;
        };

        replications.prototype.createNewDestination = function () {
            this.replicationsSetup().destinations.unshift(replicationDestination.empty());
        };

        replications.prototype.removeDestination = function (repl) {
            this.replicationsSetup().destinations.remove(repl);
        };

        replications.prototype.toggleUserCredentials = function (destination) {
            destination.isUserCredentials.toggle();
        };

        replications.prototype.toggleApiKeyCredentials = function (destination) {
            destination.isApiKeyCredentials.toggle();
        };

        replications.prototype.saveChanges = function () {
            var _this = this;
            if (this.replicationsSetup().source()) {
                this.saveReplicationSetup();
            } else {
                var db = this.activeDatabase();
                if (db) {
                    new getDatabaseStatsCommand(db).execute().done(function (result) {
                        _this.prepareAndSaveReplicationSetup(result.DatabaseId);
                    });
                }
            }
        };

        replications.prototype.prepareAndSaveReplicationSetup = function (source) {
            this.replicationsSetup().source(source);
            this.saveReplicationSetup();
        };

        replications.prototype.saveReplicationSetup = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                new saveReplicationDocumentCommand(this.replicationsSetup().toDto(), db).execute().done(function () {
                    return _this.replicationsSetupDirtyFlag().reset();
                });
            }
        };

        replications.prototype.saveAutomaticConflictResolutionSettings = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                new saveAutomaticConflictResolutionDocument(this.replicationConfig().toDto(), db).execute().done(function () {
                    return _this.replicationConfigDirtyFlag().reset();
                });
            }
        };
        return replications;
    })(viewModelBase);

    
    return replications;
});
