var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "plugins/router", "durandal/app", "durandal/system", "models/database", "common/appUrl", "models/collection", "common/alertType", "commands/getDatabaseStatsCommand", "commands/getDatabasesCommand", "commands/getBuildVersionCommand", "commands/getLicenseStatusCommand", "common/dynamicHeightBindingHandler", "common/autoCompleteBindingHandler", "viewmodels/viewModelBase", "commands/getDocumentsMetadataByIDPrefixCommand", "commands/getDocumentWithMetadataCommand", "common/changesApi", "commands/filesystem/getFilesystemsCommand", "commands/filesystem/getFilesystemStatsCommand"], function(require, exports, router, app, sys, database, appUrl, collection, alertType, getDatabaseStatsCommand, getDatabasesCommand, getBuildVersionCommand, getLicenseStatusCommand, dynamicHeightBindingHandler, autoCompleteBindingHandler, viewModelBase, getDocumentsMetadataByIDPrefixCommand, getDocumentWithMetadataCommand, changesApi, getFilesystemsCommand, getFilesystemStatsCommand) {
    var shell = (function (_super) {
        __extends(shell, _super);
        function shell() {
            var _this = this;
            _super.call(this);
            this.router = router;
            this.databases = ko.observableArray();
            this.currentAlert = ko.observable();
            this.buildVersion = ko.observable();
            this.licenseStatus = ko.observable();
            this.recordedErrors = ko.observableArray();
            this.newIndexUrl = appUrl.forCurrentDatabase().newIndex;
            this.newTransformerUrl = appUrl.forCurrentDatabase().newTransformer;
            this.filesystems = ko.observableArray();
            this.canShowFilesystemNavbar = ko.computed(function () {
                return _this.filesystems().length > 0 && _this.appUrls.isAreaActive('filesystems');
            });
            this.currentRawUrl = ko.observable("");
            this.rawUrlIsVisible = ko.computed(function () {
                return _this.currentRawUrl().length > 0;
            });
            this.activeArea = ko.observable("Databases");
            this.goToDocumentSearch = ko.observable();
            this.goToDocumentSearchResults = ko.observableArray();
            ko.postbox.subscribe("Alert", function (alert) {
                return _this.showAlert(alert);
            });
            ko.postbox.subscribe("ActivateDatabaseWithName", function (databaseName) {
                return _this.activateDatabaseWithName(databaseName);
            });
            ko.postbox.subscribe("ActivateFilesystemWithName", function (filesystemName) {
                return _this.activateFilesystemWithName(filesystemName);
            });
            ko.postbox.subscribe("SetRawJSONUrl", function (jsonUrl) {
                return _this.currentRawUrl(jsonUrl);
            });
            ko.postbox.subscribe("ActivateDatabase", function (db) {
                return _this.updateChangesApi(db);
            });
            ko.postbox.subscribe("UploadFileStatusChanged", function (uploadStatus) {
                return _this.uploadStatusChanged(uploadStatus);
            });

            this.appUrls = appUrl.forCurrentDatabase();
            this.goToDocumentSearch.throttle(250).subscribe(function (search) {
                return _this.fetchGoToDocSearchResults(search);
            });
            dynamicHeightBindingHandler.install();
            autoCompleteBindingHandler.install();
        }
        shell.prototype.activate = function (args) {
            var _this = this;
            _super.prototype.activate.call(this, args);

            NProgress.set(.7);
            router.map([
                { route: ['', 'databases'], title: 'Databases', moduleId: 'viewmodels/databases', nav: true, hash: this.appUrls.databasesManagement },
                { route: 'databases/documents', title: 'Documents', moduleId: 'viewmodels/documents', nav: true, hash: this.appUrls.documents },
                { route: 'databases/conflicts', title: 'Conflicts', moduleId: 'viewmodels/conflicts', nav: true, hash: this.appUrls.conflicts },
                { route: 'databases/patch', title: 'Patch', moduleId: 'viewmodels/patch', nav: true, hash: this.appUrls.patch },
                { route: 'databases/indexes*details', title: 'Indexes', moduleId: 'viewmodels/indexesShell', nav: true, hash: this.appUrls.indexes },
                { route: 'databases/transformers*details', title: 'Transformers', moduleId: 'viewmodels/transformersShell', nav: false, hash: this.appUrls.transformers },
                { route: 'databases/query*details', title: 'Query', moduleId: 'viewmodels/queryShell', nav: true, hash: this.appUrls.query(null) },
                { route: 'databases/tasks*details', title: 'Tasks', moduleId: 'viewmodels/tasks', nav: true, hash: this.appUrls.tasks },
                { route: 'databases/settings*details', title: 'Settings', moduleId: 'viewmodels/settings', nav: true, hash: this.appUrls.settings },
                { route: 'databases/status*details', title: 'Status', moduleId: 'viewmodels/status', nav: true, hash: this.appUrls.status },
                { route: 'databases/edit', title: 'Edit Document', moduleId: 'viewmodels/editDocument', nav: false },
                { route: ['', 'filesystems'], title: 'File Systems', moduleId: 'viewmodels/filesystem/filesystems', nav: true, hash: this.appUrls.filesystemsManagement },
                { route: 'filesystems/files', title: 'Files', moduleId: 'viewmodels/filesystem/filesystemFiles', nav: true, hash: this.appUrls.filesystemFiles },
                { route: 'filesystems/search', title: 'Search', moduleId: 'viewmodels/filesystem/search', nav: true, hash: this.appUrls.filesystemSearch },
                { route: 'filesystems/synchronization', title: 'Synchronization', moduleId: 'viewmodels/filesystem/filesystemSynchronization', nav: true, hash: this.appUrls.filesystemSynchronization },
                { route: 'filesystems/configuration', title: 'Configuration', moduleId: 'viewmodels/filesystem/configuration', nav: true, hash: this.appUrls.filesystemConfiguration },
                { route: 'filesystems/upload', title: 'Upload File', moduleId: 'viewmodels/filesystem/filesystemUploadFile', nav: false },
                { route: 'filesystems/edit', title: 'Upload File', moduleId: 'viewmodels/filesystem/filesystemEditFile', nav: false }
            ]).buildNavigationModel();

            router.isNavigating.subscribe(function (isNavigating) {
                return _this.showNavigationProgress(isNavigating);
            });
            this.connectToRavenServer();
        };

        shell.prototype.attached = function () {
            var _this = this;
            jwerty.key("ctrl+alt+n", function (e) {
                e.preventDefault();
                _this.newDocument();
            });

            $("body").tooltip({
                delay: { show: 1000, hide: 100 },
                container: 'body',
                selector: '.use-bootstrap-tooltip',
                trigger: 'hover'
            });

            router.activeInstruction.subscribe(function (val) {
                if (val.config.route.split('/').length == 1)
                    _this.activeArea(val.config.title);
            });

            shell.globalChangesApi = new changesApi(appUrl.getSystemDatabase());

            this.databasesChangeSubscription = shell.globalChangesApi.watchDocPrefix(function (e) {
                if (!!e.Id && e.Id.indexOf("Raven/Databases") == 0 && (e.Type == 1 /* Put */ || e.Type == 2 /* Delete */)) {
                    if (e.Type == 2 /* Delete */) {
                        var deletedDbName = e.Id.substring(16, e.Id.length);

                        _this.databases.remove(function (dbToRemove) {
                            return dbToRemove.name === deletedDbName;
                        });
                    }
                    _this.modelPolling();
                }
            }, "Raven/Databases");
        };

        shell.prototype.showNavigationProgress = function (isNavigating) {
            if (isNavigating) {
                NProgress.start();

                var currentProgress = parseFloat(NProgress.status);
                var newProgress = isNaN(currentProgress) ? 0.5 : currentProgress + (currentProgress / 2);
                NProgress.set(newProgress);
            } else {
                NProgress.done();
                this.activeArea(appUrl.checkIsAreaActive("filesystems") ? "File Systems" : "Databases");
                $('.tooltip.fade').remove();
            }
        };

        shell.prototype.databasesLoaded = function (databases) {
            var systemDatabase = new database("<system>");
            systemDatabase.isSystem = true;
            systemDatabase.isVisible(false);
            this.databases(databases.concat([systemDatabase]));
            if (this.databases().length == 1) {
                systemDatabase.activate();
            } else {
                this.databases.first(function (x) {
                    return x.isVisible();
                }).activate();
            }
        };

        shell.prototype.filesystemsLoaded = function (filesystems) {
            this.filesystems(filesystems);
            if (this.filesystems().length != 0) {
                this.filesystems.first(function (x) {
                    return x.isVisible();
                }).activate();
            }
        };

        shell.prototype.launchDocEditor = function (docId, docsList) {
            var editDocUrl = appUrl.forEditDoc(docId, docsList ? docsList.collectionName : null, docsList ? docsList.currentItemIndex() : null, this.activeDatabase());
            this.navigate(editDocUrl);
        };

        shell.prototype.connectToRavenServer = function () {
            var _this = this;
            this.databasesLoadedTask = new getDatabasesCommand().execute().fail(function (result) {
                return _this.handleRavenConnectionFailure(result);
            }).done(function (results) {
                _this.databasesLoaded(results);
                _this.fetchStudioConfig();
                _this.fetchBuildVersion();
                _this.fetchLicenseStatus();
                router.activate();
            });

            this.filesystemsLoadedTask = new getFilesystemsCommand().execute().fail(function (result) {
                return _this.handleRavenConnectionFailure(result);
            }).done(function (results) {
                _this.filesystemsLoaded(results);
                _this.fetchBuildVersion();
                _this.fetchLicenseStatus();
            });
        };

        shell.prototype.fetchStudioConfig = function () {
            new getDocumentWithMetadataCommand("Raven/StudioConfig", appUrl.getSystemDatabase()).execute().done(function (doc) {
                appUrl.warnWhenUsingSystemDatabase = doc["WarnWhenUsingSystemDatabase"];
            });
        };

        shell.prototype.handleRavenConnectionFailure = function (result) {
            var _this = this;
            NProgress.done();
            sys.log("Unable to connect to Raven.", result);
            var tryAgain = 'Try again';
            var messageBoxResultPromise = app.showMessage("Couldn't connect to Raven. Details in the browser console.", ":-(", [tryAgain]);
            messageBoxResultPromise.done(function (messageBoxResult) {
                if (messageBoxResult === tryAgain) {
                    NProgress.start();
                    _this.connectToRavenServer();
                }
            });
        };

        shell.prototype.showAlert = function (alert) {
            var _this = this;
            if (alert.type === 3 /* danger */ || alert.type === 2 /* warning */) {
                this.recordedErrors.unshift(alert);
            }

            var currentAlert = this.currentAlert();
            if (currentAlert) {
                this.queuedAlert = alert;
                this.closeAlertAndShowNext(currentAlert);
            } else {
                this.currentAlert(alert);
                var fadeTime = 2000;
                if (alert.type === 3 /* danger */ || alert.type === 2 /* warning */) {
                    fadeTime = 4000;
                }
                setTimeout(function () {
                    return _this.closeAlertAndShowNext(alert);
                }, fadeTime);
            }
        };

        shell.prototype.closeAlertAndShowNext = function (alertToClose) {
            var _this = this;
            var alertElement = $('#' + alertToClose.id);
            if (alertElement.length === 0) {
                return;
            }

            if (alertElement.is(":hover")) {
                setTimeout(function () {
                    return _this.closeAlertAndShowNext(alertToClose);
                }, 1000);
            } else {
                alertElement.alert('close');
            }
        };

        shell.prototype.onAlertHidden = function () {
            this.currentAlert(null);
            var nextAlert = this.queuedAlert;
            if (nextAlert) {
                this.queuedAlert = null;
                this.showAlert(nextAlert);
            }
        };

        shell.prototype.newDocument = function () {
            this.launchDocEditor(null);
        };

        shell.prototype.activateDatabaseWithName = function (databaseName) {
            var _this = this;
            if (this.databasesLoadedTask) {
                this.databasesLoadedTask.done(function () {
                    var matchingDatabase = _this.databases().first(function (d) {
                        return d.name == databaseName;
                    });
                    if (matchingDatabase && _this.activeDatabase() !== matchingDatabase) {
                        ko.postbox.publish("ActivateDatabase", matchingDatabase);
                    }
                });
            }
        };

        shell.prototype.activateFilesystemWithName = function (filesystemName) {
            var _this = this;
            if (this.filesystemsLoadedTask) {
                this.filesystemsLoadedTask.done(function () {
                    var matchingFilesystem = _this.filesystems().first(function (d) {
                        return d.name == filesystemName;
                    });
                    if (matchingFilesystem && _this.activeFilesystem() !== matchingFilesystem) {
                        ko.postbox.publish("ActivateFilesystem", matchingFilesystem);
                    }
                });
            }
        };

        shell.prototype.updateChangesApi = function (newDb) {
            if (shell.currentDbChangesApi()) {
                shell.currentDbChangesApi().dispose();
            }
            shell.currentDbChangesApi(new changesApi(newDb));
        };

        shell.prototype.modelPolling = function () {
            var _this = this;
            new getDatabasesCommand().execute().done(function (results) {
                ko.utils.arrayForEach(results, function (result) {
                    var existingDb = _this.databases().first(function (d) {
                        return d.name == result.name;
                    });
                    if (!existingDb) {
                        _this.databases.unshift(result);
                    }
                });
            });

            new getFilesystemsCommand().execute().done(function (results) {
                ko.utils.arrayForEach(results, function (result) {
                    var existingFs = _this.filesystems().first(function (d) {
                        return d.name == result.name;
                    });
                    if (!existingFs) {
                        _this.filesystems.unshift(result);
                    }
                });
            });

            var db = this.activeDatabase();
            if (db) {
                new getDatabaseStatsCommand(db).execute().done(function (result) {
                    return db.statistics(result);
                });
            }

            var fs = this.activeFilesystem();
            if (fs) {
                new getFilesystemStatsCommand(fs).execute().done(function (result) {
                    return fs.statistics(result);
                });
            }
        };

        shell.prototype.selectDatabase = function (db) {
            if (db.name != this.activeDatabase().name) {
                db.activate();
                var updatedUrl = appUrl.forCurrentPage(db);
                this.navigate(updatedUrl);
            }
        };

        shell.prototype.selectFilesystem = function (fs) {
            if (fs.name != this.activeFilesystem().name) {
                fs.activate();
                var updatedUrl = appUrl.forCurrentPage(fs);
                this.navigate(updatedUrl);
            }
        };

        shell.prototype.goToDoc = function (doc) {
            this.goToDocumentSearch("");
            this.navigate(appUrl.forEditDoc(doc['@metadata']['@id'], null, null, this.activeDatabase()));
        };

        shell.prototype.getDocCssClass = function (doc) {
            return collection.getCollectionCssClass(doc['@metadata']['Raven-Entity-Name']);
        };

        shell.prototype.fetchBuildVersion = function () {
            var _this = this;
            new getBuildVersionCommand().execute().done(function (result) {
                return _this.buildVersion(result);
            });
        };

        shell.prototype.fetchLicenseStatus = function () {
            var _this = this;
            new getLicenseStatusCommand().execute().done(function (result) {
                return _this.licenseStatus(result);
            });
        };

        shell.prototype.fetchGoToDocSearchResults = function (query) {
            var _this = this;
            if (query.length >= 2) {
                new getDocumentsMetadataByIDPrefixCommand(query, 10, this.activeDatabase()).execute().done(function (results) {
                    if (_this.goToDocumentSearch() === query) {
                        _this.goToDocumentSearchResults(results);
                    }
                });
            } else if (query.length == 0) {
                this.goToDocumentSearchResults.removeAll();
            }
        };

        shell.prototype.showErrorsDialog = function () {
            var _this = this;
            require(["viewmodels/recentErrors"], function (ErrorDetails) {
                var dialog = new ErrorDetails(_this.recordedErrors);
                app.showDialog(dialog);
            });
        };

        shell.prototype.uploadStatusChanged = function (item) {
            var queue = this.parseUploadQueue(window.localStorage[this.localStorageUploadQueueKey + item.filesystem.name], item.filesystem);
            this.updateQueueStatus(item.id(), item.status(), queue);
            this.updateLocalStorage(queue, item.filesystem);
        };
        shell.currentDbChangesApi = ko.observable(null);
        return shell;
    })(viewModelBase);

    
    return shell;
});
