define(["require", "exports", "widgets/virtualTable/cell"], function(require, exports, cell) {
    var row = (function () {
        function row(addIdCell, viewModel) {
            this.viewModel = viewModel;
            this.top = ko.observable(0);
            this.rowIndex = ko.observable(0);
            this.isInUse = ko.observable(false);
            this.cellMap = {};
            this.collectionClass = ko.observable("");
            this.editUrl = ko.observable("");
            this.isChecked = ko.observable(false);
            if (addIdCell) {
                this.addOrUpdateCellMap('Id', null);
            }
        }
        row.prototype.resetCells = function () {
            for (var prop in this.cellMap) {
                var cellVal = this.cellMap[prop];
                cellVal.reset();
            }
            this.collectionClass('');
            this.isChecked(false);
        };

        row.prototype.createPlaceholderCells = function (cellNames) {
            var _this = this;
            cellNames.filter(function (c) {
                return c != "Id";
            }).forEach(function (c) {
                return _this.addOrUpdateCellMap(c, null);
            });
        };

        row.prototype.fillCells = function (rowData) {
            this.isInUse(true);
            var rowProperties = rowData.getDocumentPropertyNames();
            for (var i = 0; i < rowProperties.length; i++) {
                var prop = rowProperties[i];
                var cellValue = rowData[prop];

                if (typeof cellValue === "object" && this.getCellTemplateName(prop, rowData) !== cell.customTemplate) {
                    cellValue = JSON.stringify(cellValue, null, 4);
                }
                this.addOrUpdateCellMap(prop, cellValue);
            }

            if (rowData.getId()) {
                this.addOrUpdateCellMap("Id", rowData.getId());
            }
        };

        row.prototype.addOrUpdateCellMap = function (propertyName, data) {
            if (!this.cellMap[propertyName]) {
                this.cellMap[propertyName] = new cell(data, this.getCellTemplateName(propertyName, data));
            } else {
                var cellVal = this.cellMap[propertyName];
                cellVal.data(data);
            }
        };

        row.prototype.getCellData = function (cellName) {
            var cellVal = this.cellMap[cellName];
            if (cellVal) {
                return cellVal.data;
            }

            return '';
        };

        row.prototype.getCellTemplate = function (cellName) {
            var cellVal = this.cellMap[cellName];
            if (cellVal) {
                return cellVal.templateName;
            }

            return null;
        };

        row.prototype.getCellTemplateName = function (propertyName, data) {
            if (propertyName === "Id") {
                return cell.idTemplate;
            } else if (propertyName === "__IsChecked") {
                return cell.checkboxTemplate;
            } else if (!!data) {
                if (typeof data == "string") {
                    var cleanData = data.replace('/\t+/g', '').replace(/\s+/g, '').replace('/\n+/g', '');
                    if (/^\[{"[a-zA-Z0-9_-]+":/.test(cleanData) || /(http|ftp|https):\/\/[\w-]+(\.[\w-]+)+([\w.,@?^=%&amp;:\/~+#-]*[\w@?^=%&amp;\/~+#-])?/.test(cleanData))
                        return cell.defaultTemplate;
                    if (/\w+\/\w+/ig.test(data))
                        return cell.externalIdTemplate;
                } else if (!!data[propertyName] && typeof data[propertyName] == "string" && /\w+\/\w+/ig.test(data[propertyName])) {
                    return cell.externalIdTemplate;
                }
            }

            var colParam = this.viewModel.settings.customColumns().findConfigFor(propertyName);

            if (colParam && colParam.template() !== cell.defaultTemplate) {
                return cell.customTemplate;
            }

            return cell.defaultTemplate;
        };
        return row;
    })();

    
    return row;
});
